/* NAME
 *	pg_queries.h
 *
 * DESCRIPTION
 *	These are the queries used by the data dict routines. Additional 
 *	where clause predicates may get tacked on by the calling routine if
 *	just one object is being described.
 *
 *      If a query is changed in this file, be sure to update the source code!
 *	And any source code executing these queries would do well to double
 *	check the number of result columns.
 *
 *	SccsId = "@(#)dd_queries.h	5.7 8/12/93"
 */

#define COMMENT_SIZE 30

#define ALL_CLASSES\
"retrieve (p.oid, p.relname, \
     comment=s2k_get_comment(p.relname, \"class\"::char16)) \
from p in pg_class where p.relkind=\"r\" "

#define USER_CLASSES     "and (p.relname < \"pg\" or p.relname > \"ph\") "
#define SYS_CLASSES      "and p.relname ~ \"pg_*\" "

/* --------------
 * ALL_ATTRIBUTES
 * --------------
 * p.att_name   Attribute name.
 * p.attnelems  Number of elements (0=scalar, -1 = undimensioned array).
 * typname      Type name. This is a udf because jta went crazy over user
 *              defined functions. It could be switched back to a straight
 *              join to pg_types at any time.
 * comment      Attribute description; udf does a lookup on s2k_comments.
 *              It must be a udf until jta figures out how to do postquel 
 *              outer joins. If it gets switched to a straight join, any 
 *              attributes that do not have a description will drop out of
 *              the results.
 * p.attnum     column number -- we have to sort on this; otherwise pg_dd
 *              can (and has) delivered columns in the wrong order.
 * inherited    Is this attribute inherited from another class?
 *              0 is no. jta lost the battle on trying to figure out
 *              outer joins.
 */

#define ALL_ATTRIBUTES\
"retrieve (p.attname, p.attnelems, typname=gdi_get_type (p.atttypid), \
	comment=s2k_get_comment(p.attname, \"attribute\"::char16), p.attnum, \
        inherited = 0) \
from p in pg_attribute "

#define INHERITS_ATTRIBUTES\
"retrieve (p.attname, p.attnelems, typname=gdi_get_type (p.atttypid), \
	comment=s2k_get_comment(p.attname, \"attribute\"::char16), p.attnum, \
        inherited = s2k_inherits(p.attrelid, p.attname)) \
from p in pg_attribute "

#define USER_ATTRIBUTES  "where p.attnum > 0"
#define SYS_ATTRIBUTES   "where p.attnum <= 0"

#define INHERITS\
"retrieve (p.inhseqno, r.relname) \
from      p in pg_inherits, r in pg_class \
where     p.inhparent=r.oid"

/* -------------
 * USER_INDEXES 
 * -------------
 * r1.relname   index name
 * r2.relname   class the index is on
 * indexrelid   = pg_class.oid where pg_class.relkind="i"
 * indrelid     = pg_class.oid where pg_class.relkind="r"
 * indkey       indkey[1-7] = pg_attribute.attnum
 * indclass     indclass[1-7] index type (e.g., int2_ops)
 * indproc      I think this is btree, rtree, etc. (not sure)
 */

#define ALL_INDEXES\
"retrieve (r1.relname, r2.relname, \
          i.indexrelid, i.indrelid, i.indkey, i.indclass, i.indproc, \
          comment=s2k_get_comment(r1.relname, \"index\"::char16)) \
from      r1 in pg_class, r2 in pg_class, i in pg_index \
where     r1.oid = i.indexrelid and r2.oid = i.indrelid \
  and     r1.relkind = \"i\" "

#define USER_INDEXES    "and r1.relname !~ \"pg_*\""
#define SYS_INDEXES     "and r1.relname ~ \"pg_*\""

/* --------------
 * USER_FUNCTIONS
 * --------------
 * proname      Function name.
 * prolang      Language. 13 = c, 14 = postquel; anybody know what 11 is?
 * typname      Return type. udf lookups up type name from prorettype.
 * proargtypes  Array of argument types.
 * prosrc       The Postquel query if language is postquel.
 * probin       The object code file if language is C.
 *
 */

#define ALL_FUNCTIONS\
"retrieve (p.proname, p.prolang, typname=gdi_get_type (p.prorettype), \
          p.proargtypes, p.prosrc, p.probin, \
          comment=s2k_get_comment(p.proname, \"function\"::char16), \
          p.prorettype) \
from      p in pg_proc "
