/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_ArrayStructs.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for creating arrays
 *	of structures for the Generic Database Interface.
 *	This array of tuples can't be unhooked or destroyed 
 *	on a tuple-by-tuple basis.
 *
 *	The function gdi_get_ArrayStructs hides the gdi object
 *	and provides the user an array of structures.
 *	The function returns the number of rows.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_ArrayStructs.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	Jeff Given 1993
 *
 * MODIFICATIONS
 *	Mari Mortell   2/93  changed function names, implemented for gdi.
 *
 *	B. MacRitchie  6/93  added inserts, removed usage of gdi_dyn_array() 
 *                           added contr_args to every fcn, allow select lists 
 *                           in any order.
 */

#ifndef lint
static char SccsId[] = "@(#)gdi_ArrayStructs.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

#include <stdio.h>
#include <math.h>
#include <string.h>

#include "libgendb.h"
#include "utilities.h"
#include "gdi_ArrayStructs.h"

Proto (int, strncasecmp, ());

#define ARRAY_INCR    50

/*
 * prototypes for arrays of structures Tuple Constructor
 */

Proto (static void *, gdi_ArrayStructs_container_create,
       (
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_ArrayStructs_container_destroy,
       (
	void	*array,	        /* (i) data container              */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static void *, gdi_ArrayStructs_tuple_create, 
       (
	dbColDef **coldef, 	/* (i) used to check select list */
	                        /*     against data structure    */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_ArrayStructs_fill_data, 
       (
	int      col_num, 	/* (i) column number             */
	dbColDef *coldef, 	/* (i) column definition (ignored) */
	void	 *tuple, 	/* (i) ArrayStructs tuple        */
	void     *dbdata, 	/* (i) database data             */
	int	 is_null, 	/* (i) "is null" flag            */
	int      str_length,	/* (i) string length             */
	int      arr_length,	/* (i) array length              */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static void *, gdi_ArrayStructs_get_data, 
       (
	int      col_num, 	/* (i) column number             */
	dbColDef *coldef, 	/* (i) column definition (ignored) */
	void	 *tuple, 	/* (i) ArrayStructs tuple        */
	int	 *is_null, 	/* (o) "is null" flag            */
	int      *str_length,	/* (o) data length               */
	int      *arr_length,	/* (o) data length               */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_ArrayStructs_tuple_add, 
       (
	void  **array,		/* (i) pointer to ArrayStructs data container */
	void  *tuple, 		/* (i) ArrayStructs tuple          */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static void *, gdi_ArrayStructs_tuple_retrieve, 
       (
	void  *array, 		/* (i) data container  */
	int   tuple_num,	/* (i) tuple number    */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static void, gdi_ArrayStructs_data_destroy,
       (
	void  *array,		/* (i) ArrayStructs data container */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_check_data_types, 
       (
	dbColDef **coldef,      /* (i) col defs from dbObj */
	ArrayStructsArgs **args /* (i) pointer to constructor arguments */
	));


#ifdef __STDC__
dbConstr GDI_ARRAYSTRUCTS =
{
     (int    (*) (void *, void **))          gdi_ArrayStructs_container_destroy,
     (void * (*) (void **))                  gdi_ArrayStructs_container_create,
     (int    (*) (void **, void *, void **)) gdi_ArrayStructs_tuple_add,
     (void * (*) (void *, int, void **))     gdi_ArrayStructs_tuple_retrieve,
     (int    (*) ())                         NULL,
     (void * (*) (dbColDef **, void **))     gdi_ArrayStructs_tuple_create,
     (int    (*) (int, dbColDef *, void *, void *, int, int, int, void **))
	                                     gdi_ArrayStructs_fill_data,
     (void * (*) (int, dbColDef *, void *, int *, int *, int *, void **)) 
	                                     gdi_ArrayStructs_get_data,
     (int    (*) ())                         NULL,
     (void * )        NULL,
     (dbCoerce)       GDI_DEFAULT_COERCE
};
#else
dbConstr GDI_ARRAYSTRUCTS =
{
     (int    (*) ())  gdi_ArrayStructs_container_destroy,
     (void * (*) ())  gdi_ArrayStructs_container_create,
     (int    (*) ())  gdi_ArrayStructs_tuple_add,
     (void * (*) ())  gdi_ArrayStructs_tuple_retrieve,
     (int    (*) ())  NULL,
     (void * (*) ())  gdi_ArrayStructs_tuple_create,
     (int    (*) ())  gdi_ArrayStructs_fill_data,
     (void * (*) ())  gdi_ArrayStructs_get_data,
     (int    (*) ())  NULL,
     (void *)         NULL,
     (dbCoerce)       GDI_DEFAULT_COERCE
};
#endif


/*
 *   This package allows contruction of one array/submit
 *
 *   The description of the container for the array is maintained in **args.  
 *   This description changes for each type of array being retrieved.
 */

static 
void *
gdi_ArrayStructs_container_create (args)
ArrayStructsArgs **args;
{
     char	      *routine = "gdi_ArrayStructs_container_create: ";
     ArrayStructsArgs *new_args;
     void             *new_container;
     
     if ((args == NULL) || (*args == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_ArrayStructs_container_create: NULL args");
	  return (NULL);
     }

     /*
      * tricky... We want to use the args to pass flags and data between the
      * various constructor functions.  So, we need to replace the args with
      * a COPY so that we can maintain a separate set of arg structures for
      * each dbObj built by the GDI library.
      */
     
     if ((new_args = UCALLOC (ArrayStructsArgs, 1)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (NULL);
     }

     bcopy ((*args), new_args, sizeof (ArrayStructsArgs));

     new_args -> nrows = 0;

     *args = new_args;
     
     /* 
      * alloc a blank tuple
      */
     if (((*args)->blank_tuple = (void *) calloc (1, (unsigned) (*args)->size)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (NULL);
     }
     
     /* 
      * alloc a container
      */
     if ((new_container = (void *) calloc (ARRAY_INCR, (unsigned) (*args)->size)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (NULL);
     }

     return (new_container);
}


/*
 *  This function destroys the container, the data and the dbConstr args
 */
static 
int
gdi_ArrayStructs_container_destroy (array, args)
void             *array;
ArrayStructsArgs **args;
{
     if ((args == NULL) || (*args == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_ArrayStructs_container_destroy: NULL args");
	  return (NULL);
     }

     if (array != NULL)
     {
	  gdi_ArrayStructs_data_destroy (array, args);
	  free (array);
     }
     
     /*
      * deallocate args
      */
     if (*args) 
     {
	  UFREE ((*args)->col_nums);
	  UFREE ((*args)->blank_tuple);
	  UFREE (*args);
     }

     return (GDI_SUCCESS);
}



/*
 *     return a new tuple.  For array inserts this is a static
 *     piece of memory that array_add() will add to its array.
 *     
 *     Initialize it to the null_tuple as defined for the current
 *     container
 */

static 
void *
gdi_ArrayStructs_tuple_create (coldef, args)
dbColDef         **coldef;
ArrayStructsArgs **args;
{
     if ((args == NULL) || (*args == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_ArrayStructs_tuple_create: NULL args");
	  return (NULL);
     }

     /*
      * Make sure the user's column defs (and => select list)
      * match the data structures.
      */
     if ((*args)->col_nums == NULL)
	  if (gdi_check_data_types (coldef, args) == GDI_FAILURE)
	       return (NULL);

     
     bcopy ((*args)->null_tuple, 
	    (*args)->blank_tuple,
	    (*args)->size);
     
     return ((*args)->blank_tuple);
}


/*
 *       I haven't tested this one, yet.
 */

static 
void *
gdi_ArrayStructs_tuple_retrieve (array, n, args)
void  *array;
int   n;
ArrayStructsArgs **args;
{
     if (array == NULL)
	  return((void *) NULL);
     
     if ((args == NULL) || ((*args) == NULL))
	  return (NULL);
     
     return ((void *) ((char *) array + ((*args)->size * n)));
}


/*
 *    Add a tuple to the container
 */

static 
int
gdi_ArrayStructs_tuple_add (array, tuple, args)
void  **array;
void  *tuple;
ArrayStructsArgs **args;
{
     int   size;
     int   len;
     void  *tmp_array;
     char  *routine = "gdi_ArrayStructs_tuple_add: ";
     
     if ((args == NULL) || (*args == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA, "gdi_ArrayStructs_tuple_add: NULL args");
	  return (NULL);
     }

     size = (*args)->size;
     len  = (*args)->nrows;
     
     if (((len + 1) % ARRAY_INCR) == 0)
     {
	  if ((tmp_array = realloc ((*array), size * (len + ARRAY_INCR))) == NULL)
	  {
	       gdi_error_unix (NULL, routine);
	       return (GDI_FAILURE);
	  }
	  (*array) = tmp_array;
     }
     
     bcopy (tuple, (char *) (*array) + (len * size), size);

     (*args)->nrows = len + 1;

     return (GDI_SUCCESS);
}


/*
 * Put a data element in a a tuple
 */
static 
int
gdi_ArrayStructs_fill_data (col_num, coldef, tuple, dbdata, is_null, 
			    str_length, arr_length, args)
int	 col_num;
dbColDef *coldef;
void	 *tuple;
void	 *dbdata;
int	 is_null;
int	 str_length;
int	 arr_length;
ArrayStructsArgs **args;
{
     ElemAttr *attr;
     int      boff;
     Elem_t   type;
     int      size;
     int      field_num;
     char     *routine = "gdi_ArrayStructs_fill_data: ";
     
     if ((args == NULL) || (*args == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_ArrayStructs_fill_data: NULL args");
	  return (NULL);
     }

     if (col_num < 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_ArrayStructs_fill_data: Invalid column number");
	  return (GDI_FAILURE);
     }

     if (arr_length > 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_ArrayStructs_fill_data: Can't handle array data");
	  return (GDI_FAILURE);
     }

     /*
      * Get field number from col_nums array
      */
     field_num = (*args)->col_nums [col_num];
     attr = &(((*args)->elem_attr) [field_num]);
     
     boff = attr->offset;
     size = attr->size;
     type = attr->type;
     
     if ((tuple == NULL) || (dbdata == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_ArrayStructs_fill_data: NULL pointer argument");
	  return (GDI_FAILURE);
     }
     
     bzero ((char *) tuple + boff, size);
     
     /*
      *   If this field is NULL, then we're done.
      */
     if (is_null)
     {
	  return (GDI_SUCCESS);
     }
     
     switch (type)
     {
     case carray_t:
	  bcopy (dbdata, (char *) tuple + boff, 
		 MIN (size - 1, str_length + 1) * sizeof (char));
	  ((char *) ((char *) tuple + boff)) [size - 1] = '\0';
	  break;
     case int_t:
	  *(int *) ((char *) tuple + boff)    = * ((int *) dbdata);
	  break;
     case long_t:
	  *(long *) ((char *) tuple + boff)   = * ((long *) dbdata);
	  break;
     case double_t:
	  *(double *) ((char *) tuple + boff) = * ((double *) dbdata);
	  break;
     case float_t:
	  *(float *) ((char *) tuple + boff)  = * ((float *) dbdata);
	  break;
     case cptr_t:
	  if ((* (char **) ((char *) tuple + boff) = UCALLOC (char, str_length + 1)) 
	      == NULL)
	  {
	       gdi_error_unix (NULL, routine);
	       return (GDI_FAILURE);
	  }
	  bcopy ((char *) dbdata, * (char **) ((char *) tuple + boff), 
		 str_length * sizeof (char));
	  (* (char **) ((char *) tuple + boff)) [str_length] = '\0';
	  break;
     default:
	  break;
     }
     return (GDI_SUCCESS);
}

/*
 * Get a data element from a tuple
 */
static
void *
gdi_ArrayStructs_get_data (col_num, coldef, tuple, is_null, 
			   str_length, arr_length, args)
int	 col_num;
dbColDef *coldef;
void	 *tuple;
int	 *is_null;
int	 *str_length;
int	 *arr_length;
ArrayStructsArgs **args;
{
     ElemAttr *attr;
     int      boff;
     Elem_t   type;
     int      size;
     void     *dbdata = NULL;

     if ((is_null == NULL) || (str_length == NULL) || (arr_length == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_ArrayStructs_get_data: NULL pointer args");
	  return (NULL);
     }

     *is_null = FALSE;
     *str_length = 0;
     *arr_length = GDI_SCALAR;

     if ((args == NULL) || (*args == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_ArrayStructs_get_data: NULL args");
	  return (NULL);
     }

     attr = &(((*args)->elem_attr) [col_num]);
     
     boff = attr->offset;
     size = attr->size;
     type = attr->type;
     
     if (col_num < 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_ArrayStructs_get_data: Invalid column number");
	  return (NULL);
     }
     
     if (tuple == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_ArrayStructs_get_data: NULL tuple");
	  return (NULL);
     }
     
     switch (type)
     {
     case carray_t:
	  dbdata = (void *) ((char *) tuple + boff);
	  *str_length = size;
	  break;
     case int_t:
     case long_t:
     case double_t:
     case float_t:
	  dbdata = (void *) ((char *) tuple + boff);
	  break;
     case cptr_t:
	  dbdata = (void *) (* (char **) ((char *) tuple + boff));
	  *str_length = strlen ((char *) dbdata);
	  break;
     default:
	  break;
     }

     return (dbdata);
}


/*
 *  Destroy tuple data (the only thing to destroy are the string pointers
 *  in the tuples since the tuples themselves cannot be separately destroyed)
 */
static 
void
gdi_ArrayStructs_data_destroy (array, args)
void  *array;
ArrayStructsArgs **args;
{
     int  i, j;
     int  n            = (*args) -> nrows;
     int  size         = (*args) -> size;
     ElemAttr *defs    = (*args) -> elem_attr;
     int  boff;

     /*
      *   This function destroys the array data
      */
     if (array == NULL)
	  return;
     
     /*
      *   Free up any char pointer data
      */
     
     for(i=0; i < n; ((char *) array) += size, i++)
     {
	  for(j = 0; defs[j].name != (char *) NULL; j++)
	  {
	       switch(defs[j].type)
	       {
	       case cptr_t:
		    boff = defs[i].offset;
		    UFREE(* (char **) ((char *) array + boff));
		    break;
	       default:
		    break;
	       }
	  }
     }
     
     return;
}



/*
 * gdi_check_data_types
 *
 *	The select list must be consistant with the data structure.
 *
 *      The select list may contain any subset of fields in the table 
 *      structure in any order.  Any fields not listed in the select
 *      list are filled with NA values.
 *
 *      This function looks for a field name matching the select list 
 *      name and checks for a matching datatype.  The results are
 *      stored in the column numbers array of the ArrayStructsArgs.
 */

static 
int
gdi_check_data_types (coldef, args)
dbColDef          **coldef;
ArrayStructsArgs  **args;
{
     int       i;
     int       j;
     int       nsel;
     ElemAttr  *attr;
     char      *routine = "gdi_check_data_types: ";
     char      *errmsg  = "%s: Data type in col %d does not match data structure.";
     char      errbuf [256];
     
     /*
      * Determine the number of elements in the select list
      */
     for (nsel = 0; coldef[nsel] != NULL; nsel++);
     
     /*
      * Create an array to hold the column numbers
      */
     if (((*args)->col_nums = UCALLOC (int, nsel)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (GDI_FAILURE);
     }
     
     attr = (*args)->elem_attr;
     
     /*
      * Search the elem_attr for a matching column name and data type.
      *
      * Column def numbers are all doubles because the GDI_DBL_STR_COERCE 
      * coerce option was used.
      */
     for (j = 0; j < nsel; j++)
     {
	  for (i = 0; (attr[i].name != NULL) &&
	       (strncasecmp (attr[i].name, coldef[j]->name, GDI_MAX_NAME) != 0); i++);
	  
	  if (attr[i].name == NULL)
	  {
	       gdi_error_app (NULL, GDI_BADDATA,
			      "gdi_check_data_types: Select list doesn't match data structure.");
	       return (GDI_FAILURE);
	  }
	  
	  switch (attr[i].type)
	  {
	  case carray_t:
	  case cptr_t:
	       if ((coldef[j]->ctype != GDI_CHAR ) && 
		   (coldef[j]->ctype != GDI_STRING) )
	       {
		    sprintf (errbuf, errmsg, routine, j);
		    gdi_error_app (NULL, GDI_BADDATA, errbuf);
		    return (GDI_FAILURE);
	       }
	       if (coldef[j]->max_strlen < attr[i].size )
	       {
		    sprintf (errbuf, errmsg, routine, j);
		    gdi_error_app (NULL, GDI_BADDATA, errbuf);
		    return (GDI_FAILURE);
	       }
	       break;
	  case int_t:
	       if (coldef[j]->ctype != GDI_INTEGER )
	       {
		    sprintf (errbuf, errmsg, routine, j);
		    gdi_error_app (NULL, GDI_BADDATA, errbuf);
		    return (GDI_FAILURE);
	       }
	       break;
	  case long_t:
	       if (coldef[j]->ctype != GDI_LONG )
	       {
		    sprintf (errbuf, errmsg, routine, j);
		    gdi_error_app (NULL, GDI_BADDATA, errbuf);
		    return (GDI_FAILURE);
	       }
	       break;
	  case float_t:
	       if (coldef[j]->ctype != GDI_FLOAT)
	       {
		    sprintf (errbuf, errmsg, routine, j);
		    gdi_error_app (NULL, GDI_BADDATA, errbuf);
		    return (GDI_FAILURE);
	       }
	       break;
	  case double_t:
	       if (coldef[j]->ctype != GDI_DOUBLE )
	       {
		    sprintf (errbuf, errmsg, routine, j);
		    gdi_error_app (NULL, GDI_BADDATA, errbuf);
		    return (GDI_FAILURE);
	       }
	       break;
	  default:
	       break;
	  }
	  
	  /*
	   * End of data structure, could not find the select list
	   * item in the attribute definitions.
	   */
	  
	  (*args)->col_nums [j] = i;
     }
     
     return (GDI_SUCCESS);
}



/*
 *   Generic tuple array fetch
 *
 *	This hides the gdi obj implementation from the ArrayStructs user.  
 *      All they see is the array of tuples.
 *
 *   Returns -1 if error, ntuples if ok.
 *
 */

int
gdi_get_ArrayStructs (dbconn, query, array, maxrec, type)
dbConn		  *dbconn;
char		  *query;
void		  **array;
int		  maxrec;
ArrayStructsArgs  *type;
{
     int       ntuples;
     dbConstr  constr;
     dbObj     *dbobj;
     char      *routine = "gdi_get_ArrayStructs: ";
     
     if (type == NULL)
     {
	  gdi_error_app (dbconn, GDI_BADDATA, 
			 "gdi_add_ArrayStructs: NULL Array Structs type");
	  return (GDI_FAILURE);
     }	  

     /* 
      * make a copy of the constructor so there is no chance of colliding with
      * other calls to libgdi
      */
     bcopy (&GDI_ARRAYSTRUCTS, &constr, sizeof (dbConstr));
     constr.args = (void *) type;
     
     if (gdi_submit (dbconn, query, maxrec, &constr, &dbobj) != GDI_SUCCESS)
     {
	  gdi_error_preface (dbconn, routine);
	  gdi_obj_destroy (dbobj);
	  return (-1);
     }

     ntuples = GDI_OBJ_NUM_TUPLES (dbobj);
     
     /*
      * Remove the tuples from the dbObj.  gdi_obj_destroy() will no longer be
      * able to destroy the tuple data.  Now, it is the responsibility of the 
      * application to manage the array of tuple structures.
      */
     *array = GDI_OBJ_TUPLE_CONTAINER (dbobj);
     GDI_OBJ_TUPLE_CONTAINER (dbobj) = NULL;
     
     if (gdi_obj_destroy (dbobj) != NULL)
     {
	  gdi_error_unix (dbconn, "gdi_get_ArrayStructs: Error in freeing dbObj");
     }
     
     return (ntuples);
}



/*
 *   Generic tuple array insert
 *
 *	This hides the gdi obj implementation from the ArrayStructs user.  
 *      All they see is the array of tuples.
 *
 *   Returns -1 if error, number of tuples inserted if ok.
 *
 */

int
gdi_add_ArrayStructs (dbconn, table_name, array, ntuples, type)
dbConn		  *dbconn;
char              *table_name;
void		  *array;
int		  ntuples;
ArrayStructsArgs  *type;
{
     dbConstr  constr;
     dbObj     *dbobj;
     int       maxrecs = 0;
     char      qbuf [300];
     char      *query = "select * from %s where 1 = 2";
     char      *routine = "gdi_add_ArrayStructs: ";
     
     if (type == NULL)
     {
	  gdi_error_app (dbconn, GDI_BADDATA, 
			 "gdi_add_ArrayStructs: NULL Array Structs type");
	  return (-1);
     }	  

     if (array == NULL)
     {
	  gdi_error_app (dbconn, GDI_BADDATA, 
			 "gdi_add_ArrayStructs: NULL Array Structs array");
	  return (-1);
     }	  

     /* 
      * make a copy of the constructor so there is no chance of colliding with
      * other calls to libgdi
      */
     bcopy (&GDI_ARRAYSTRUCTS, &constr, sizeof (dbConstr));
     constr.args = (void *) type;
     
     /*
      * have to reconstruct a dbObj matching the array structs since 
      * gdi_get_ArrayAstructs() threw it away.  Make gdi_describe_query() do it.
      */
     sprintf (qbuf, query, table_name);
     if (gdi_submit (dbconn, qbuf, maxrecs, &constr, &dbobj) != GDI_SUCCESS)
     {
 	  gdi_error_preface (dbconn, routine);
	  gdi_obj_destroy (dbobj);
	  return (-1);
     }
     /*
      * create a tuple so the constructor args are initialized properly
      */
     gdi_obj_tuple_create (dbobj);
     
     /*
      * destroy the malloc'd components of dbObj and insert a new data container
      */
     UFREE (GDI_OBJ_QUERY (dbobj));
     UFREE (GDI_OBJ_TUPLE_CONTAINER (dbobj));
     GDI_OBJ_TUPLE_CONTAINER (dbobj) = array;
     GDI_OBJ_NUM_TUPLES (dbobj) = ntuples;

     if (gdi_insert (dbconn, table_name, dbobj) != GDI_SUCCESS)
     {
	  gdi_error_preface (dbconn, routine);
	  GDI_OBJ_TUPLE_CONTAINER (dbobj) = NULL;
	  gdi_obj_destroy (dbobj);
	  return (-1);
     }

     /* 
      * destroy the dbobj but NOT the data
      */
     GDI_OBJ_TUPLE_CONTAINER (dbobj) = NULL;
     gdi_obj_destroy (dbobj);
     
     return (ntuples);
}
