/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_open(), gdi_close(), gdi_dead(), gdi_exit(),
 *	gdi_get_account(), gdi_get_database(), gdi_get_node(),
 *	gdi_add_conn(), gdi_free_conn(), gdi_create_conn()
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 * DESCRIPTION
 *	This file contains functions for managing the database connections
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	gdi_conn.c (this file)
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_conn.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


Proto (static int,	rmv_conn, (dbConn *));
Proto (static dbVendor,	vendor_id, (char *));
Proto (static int,	get_unix_id, (void));

Proto (int,     getpid, ());

extern dbConn null_conn;	/* declared in gdi_error.c */
dbConn	*conn_list = NULL;

/*
 * gdi_open ()
 *
 * Open a connection to a database
 *
 * Public
 */

dbConn *
gdi_open (vendor, account, password, database, server, appname)
char	*vendor;
char	*account;
char	*password;
char	*database;
char	*server;
char	*appname;
{
     dbConn 	*conn = NULL;
     int	status;
     char       *fname = "gdi_open: ";
     
     if( (conn = gdi_create_conn (vendor)) == NULL)
     {
	  gdi_error_preface (conn, fname);
	  return (NULL);
     }
     
     /* 
      * link in library functions for specific vendor
      */
     if (gdi_link (conn) != GDI_SUCCESS)
     {
	  gdi_error_preface (conn, fname);
	  bcopy (conn, &null_conn, sizeof (dbConn));
	  UFREE (conn);
	  return (NULL);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->open) != NULL)
     {
	  if ((status = GDI_FUNCS (conn)->open (conn, account, password, database, 
						server, appname)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
     }
     else
     {
	  status = GDI_FAILURE;
	  gdi_error_app (conn, GDI_BADVENDOR, 
			 "gdi_open: Vendor function unavailable");
     }
     
     if (status != GDI_SUCCESS)
     {
	  UFREE (conn->vendor_conn);
	  bcopy (conn, &null_conn, sizeof (dbConn));
	  UFREE (conn);
	  return (NULL);
     }
     
     /* 
      * Initialize Error Flags
      */
#if 0 /* flags already initialized to null_conn flags */
     gdi_error_init (conn, FALSE, GDI_WARNING, 0, 0);
#endif

     /* 
      * add connection to global linked list
      */
     gdi_add_conn (conn);
     
     return (conn);
}



/*
 * gdi_close ()
 *
 * Close a connection to a database
 *
 * Public
 */

int
gdi_close (conn)
dbConn		*conn;
{
     int    ret;

     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  ret = gdi_warning_app (conn, GDI_NOCONNECT,
				 "gdi_close: Not connected to database");
	  return (ret);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->close) != NULL)
     {
	  GDI_FUNCS (conn)->close (conn);
	  gdi_free_conn (conn);
	  return (GDI_SUCCESS);
     }
     
     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_close: Vendor function unavailable");

     return (GDI_FAILURE);
}



/*
 * gdi_dead ()
 *
 * Determine if a connection is dead.  Return TRUE or FALSE.
 *
 * Public
 */

int
gdi_dead (conn)
dbConn		*conn;
{
     if ((conn == NULL) || (conn->vendor_conn == NULL))
	  return (TRUE);

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->dead) != NULL)
     {
	  return (GDI_FUNCS (conn)->dead (conn));
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_dead: Vendor function unavailable");

     return (TRUE);
}


/*
 * gdi_exit ()
 *
 * Close all open database connections
 *
 * Public
 */

int
gdi_exit ()
{
     while (conn_list != NULL)
	  gdi_close (conn_list);

     return (GDI_SUCCESS);
}



/*
 * gdi_get_account ()
 *
 * Get the database account name from the database connector
 *
 * Public
 */

int
gdi_get_account (conn, account, len)
dbConn	*conn;
char	*account;
int	len;
{
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT, 
			 "gdi_get_account: Not connected to database");
	  return (GDI_FAILURE);
     }

     strncpy (account, GDI_ACCOUNT (conn), len - 1);
     account [len - 1] = '\0';
     
     return (GDI_SUCCESS);
}




/*
 * gdi_get_database ()
 *
 * Get the database name from the database connector
 *
 * Public
 */

int
gdi_get_database (conn, database, len)
dbConn	*conn;
char	*database;
int	len;
{
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT, 
			 "gdi_get_database: Not connected to database");
	  return (GDI_FAILURE);
     }

     strncpy (database, GDI_DATABASE (conn), len - 1);
     database [len - 1] = '\0';
     
     return (GDI_SUCCESS);
}




/*
 * gdi_get_node ()
 *
 * Get the database node name from the database connector
 *
 * Public
 */

int
gdi_get_node (conn, node, len)
dbConn	*conn;
char	*node;
int	len;
{
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT, 
			 "gdi_get_node: Not connected to database");
	  return (GDI_FAILURE);
     }

     strncpy (node, GDI_NODE (conn), len - 1);
     node [len - 1] = '\0';
     
     return (GDI_SUCCESS);
}


/*
 * gdi_add_conn ()
 *
 * add a dbConn to the front of conn list
 *
 * Public
 */

int
gdi_add_conn (conn)
dbConn	*conn;
{
     if (conn == NULL)
	  return (GDI_SUCCESS);
     
     conn->next_conn = conn_list;

     if (conn_list != NULL)
	  conn_list->prev_conn = conn;

     conn_list = conn;
     
     return (GDI_SUCCESS);
}


/*
 * rmv_conn ()
 *
 * remove a dbConn from the conn list
 *
 * Private
 */

static int
rmv_conn (conn)
dbConn	*conn;
{
     if (conn == NULL)
	  return (GDI_SUCCESS);
     
     if (conn == conn_list)
	  conn_list = conn->next_conn;

     if (conn->prev_conn != NULL)
	  conn->prev_conn->next_conn = conn->next_conn;
     
     if (conn->next_conn != NULL)
	  conn->next_conn->prev_conn = conn->prev_conn;

     return (GDI_SUCCESS);
}


/*
 * gdi_free_conn ()
 *
 * Free a database connection.
 *
 * Public
 */

int
gdi_free_conn (conn)
dbConn		*conn;
{
     if (conn == NULL)
	  return (GDI_SUCCESS);
     
     rmv_conn (conn);
     
     UFREE (conn->vendor_conn);
     UFREE (conn);

     return (GDI_SUCCESS);
}



/*
 * vendor_id ()
 *
 * Get vendor id from database string
 *
 * Private
 */

static dbVendor
vendor_id (vendor)
char	*vendor;
{
     if (vendor == NULL)
	  return (GDI_UNKNOWN);
     
     if (STREQL (vendor, GDI_ORACLE_S))
	  return (GDI_ORACLE);
     
     if (STREQL (vendor, GDI_POSTGRES_S))
	  return (GDI_POSTGRES);

     if (STREQL (vendor, GDI_SYBASE_S))
	  return (GDI_SYBASE);
     
     if (STREQL (vendor, GDI_INGRES_S))
	  return (GDI_INGRES);

     return (GDI_UNKNOWN);
}


dbConn *
gdi_create_conn (vendor)
char	*vendor;
{
     dbConn 	*conn;
     dbErrLev	threshold;
     dbDebug	debug;

     if ((conn = UCALLOC (dbConn, 1)) == NULL)
     {
	  (void) gdi_error_unix (NULL, 
				"gdi_open: Unable to allocate connection");
	  return (NULL);
     }

     /* 
      * Temporarily initialize error flags to null_conn error flags
      */
     gdi_error_flags (NULL, &debug, &threshold);
     gdi_error_init (conn, debug, threshold, 0, 0);

     /*
      * set vendor id
      */
	conn->vendor_id = vendor_id (vendor);

     /*
      * set operating system id
      */
	conn->os_pid = get_unix_id ();

	return(conn);
}

static int
get_unix_id ()
{
	return (getpid ());
}
