/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *
 *	gdi_constr.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for accessing the dbConstr of the
 *	Generic Database Interface.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_constr.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_constr.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif



/*
 * gdi_obj_container_create ()
 *
 * Create a new tuple container using the the container_create
 * function in the database object constructor.  Add the 
 * container to the object.  If the container_create function
 * does not exist, set container to NULL.
 *
 * Public
 */

int
gdi_obj_container_create (dbobj)
dbObj	*dbobj;
{
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_container_create: NULL dbObj");
	  return (GDI_FAILURE);
     }

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> container_create == NULL)
	  GDI_OBJ_TUPLE_CONTAINER (dbobj) = NULL;
     else
	  GDI_OBJ_TUPLE_CONTAINER (dbobj) = GDI_OBJ_CONSTRUCTOR (dbobj) ->
	       container_create (GDI_OBJ_CONSTR_ARGS (dbobj));

     return (GDI_SUCCESS);
}



/*
 * gdi_obj_container_destroy ()
 *
 * Destroy the tuple container in the dbObj using the 
 * container_destroy function in the database object constructor.
 * If the container_destroy function does not exist, use free().
 *
 * Public
 */

int
gdi_obj_container_destroy (dbobj)
dbObj	*dbobj;
{
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_container_destroy: NULL dbObj");
	  return (GDI_FAILURE);
     }

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> container_destroy == NULL)
     {
	  UFREE (GDI_OBJ_TUPLE_CONTAINER (dbobj));
	  return (GDI_SUCCESS);
     }
     
     GDI_OBJ_CONSTRUCTOR (dbobj) -> 
	  container_destroy (GDI_OBJ_TUPLE_CONTAINER (dbobj), 
			     GDI_OBJ_CONSTR_ARGS (dbobj));
     
     GDI_OBJ_TUPLE_CONTAINER (dbobj) = NULL;

     return (GDI_SUCCESS);
}



/*
 * gdi_obj_tuple_create ()
 *
 * Create a tuple using the tuple_create function in the database
 * object constructor and the coldefs in the dbObj.
 *
 * Return NULL on error.
 *
 * Public
 */

void *
gdi_obj_tuple_create (dbobj)
dbObj	*dbobj;
{
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_tuple_create: NULL dbObj");
	  return (NULL);
     }

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> tuple_create == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_tuple_create: No tuple_create function");
	  return (NULL);
     }

     return (GDI_OBJ_CONSTRUCTOR (dbobj) ->
	     tuple_create (GDI_OBJ_COL_DEFS (dbobj), GDI_OBJ_CONSTR_ARGS (dbobj)));
     
}




/*
 * gdi_obj_tuple_destroy ()
 *
 * Destroy a tuple using the tuple_destroy function in the database
 * object constructor.  If the tuple_destroy function does not
 * exist assume that the tuple is not "destroyable", so do nothing.
 *
 * Return GDI_FAILURE on error, else return GDI_SUCCESS.
 *
 * Public
 */

int
gdi_obj_tuple_destroy (dbobj, tuple)
dbObj	*dbobj;
void	*tuple;
{
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_tuple_destroy: NULL dbObj");
	  return (GDI_SUCCESS);
     }

     if (tuple == NULL)
	  return (GDI_SUCCESS);

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> tuple_destroy == NULL)
	  return (GDI_SUCCESS);

     return (GDI_OBJ_CONSTRUCTOR (dbobj) -> 
	     tuple_destroy (tuple, GDI_OBJ_CONSTR_ARGS (dbobj)));
}




/*
 * gdi_obj_tuple_add ()
 *
 * Add a tuple to a dbobj using the tuple_add function in the
 * database object constructor.
 *
 * Return GDI_FAILURE if error, else return GDI_SUCCESS.
 *
 * Public
 */

int
gdi_obj_tuple_add (dbobj, tuple)
dbObj	*dbobj;
void	*tuple;
{
     int	ret;
     
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_tuple_add: NULL dbObj");
	  return (GDI_FAILURE);
     }

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> tuple_add == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_tuple_add: No tuple_add function");
	  return (GDI_FAILURE);
     }

     ret = GDI_OBJ_CONSTRUCTOR (dbobj) ->
	  tuple_add (&GDI_OBJ_TUPLE_CONTAINER (dbobj), tuple, 
		     GDI_OBJ_CONSTR_ARGS (dbobj));
     
     if (ret == GDI_SUCCESS)
	  GDI_OBJ_NUM_TUPLES (dbobj)++;

     return (ret);
}




/*
 * gdi_obj_tuple_retrieve ()
 *
 * Get a specified tuple from the database
 * Return NULL if not found.
 *
 * Public
 */

void *
gdi_obj_tuple_retrieve (dbobj, tuple_num)
dbObj	*dbobj;
int	tuple_num;
{
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_tuple_retrieve: NULL dbObj");
	  return (NULL);
     }

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> tuple_retrieve == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_tuple_retrieve: No tuple_retrieve function");
	  return (NULL);
     }

     return (GDI_OBJ_CONSTRUCTOR (dbobj) ->
	     tuple_retrieve (GDI_OBJ_TUPLE_CONTAINER (dbobj), tuple_num,
			     GDI_OBJ_CONSTR_ARGS (dbobj)));
}




/*
 * gdi_obj_fill_data ()
 *
 * Fill a tuple with data and flags using the fill data function
 * from a database object constructor.
 *
 * Return GDI_FAILURE if not successful, otherwise return GDI_SUCCESS.
 *
 * Public
 */

int
gdi_obj_fill_data (dbobj, col_num, tuple, dbdata, is_null, str_length, arr_length)
dbObj	*dbobj;
int	col_num;
void	*tuple;
void	*dbdata;
int	is_null;
int	str_length;
int	arr_length;
{
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_fill_data: NULL dbObj");
	  return (GDI_FAILURE);
     }

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> fill_data == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_fill_data: No fill_data function");
	  return (GDI_FAILURE);
     }

     return (GDI_OBJ_CONSTRUCTOR (dbobj) ->
	     fill_data (col_num, GDI_OBJ_COL_DEF (dbobj, col_num),
			tuple, dbdata, is_null, str_length, arr_length,
			GDI_OBJ_CONSTR_ARGS (dbobj)));
}



/*
 * gdi_obj_get_data ()
 *
 * Get a pointer to a specified column value from a tuple using the
 * database object constructor function, get_data.
 * Return NULL if not found.
 *
 * Public
 */

void *
gdi_obj_get_data (dbobj, col_num, tuple, is_null, str_length, arr_length)
dbObj	*dbobj;
int	col_num;
void	*tuple;
int	*is_null;
int	*str_length;
int	*arr_length;
{
     
     if (dbobj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_get_data: NULL dbObj");
	  return (NULL);
     }

     if (GDI_OBJ_CONSTRUCTOR (dbobj) -> get_data == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_obj_get_data: No get_data function");
	  return (NULL);
     }

     return (GDI_OBJ_CONSTRUCTOR (dbobj) ->
	     get_data (col_num, GDI_OBJ_COL_DEF (dbobj, col_num),
		       tuple, is_null, str_length, arr_length,
		       GDI_OBJ_CONSTR_ARGS (dbobj)));
}






