/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_dates.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for setting and retrieving date masks.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_dates.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	Mari Mortell
 */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <sys/time.h>		/* used by gdi_get_date_now */
#include "utilities.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_dates.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

Proto (long,    time, ());


/*
 * gdi_set_date_mask ()
 *
 * Set a date mask
 *
 * Public
 */

int
gdi_set_date_mask (conn, new_mask)
dbConn		*conn;
char		*new_mask;
{
     int    ret;
     char   *fname = "gdi_set_date_mask: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT, 
			"gdi_set_date_mask: Database not connected");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->set_date_mask) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->set_date_mask (conn, new_mask)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_set_date_mask: Vendor function unavailable");

     return (GDI_FAILURE);
}


/*
 * gdi_get_date_mask ()
 *
 * Get a date mask
 *
 * Public
 */

int
gdi_get_date_mask (conn, mask_holder, holder_size)
dbConn		*conn;
char		*mask_holder;
int		holder_size;
{
     int    ret;
     char   *fname = "gdi_get_date_mask: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT, 
			"gdi_get_date_mask: Database not connected");
	  return (GDI_FAILURE);
     }
     
     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->get_date_mask) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->get_date_mask (conn, mask_holder, holder_size))
	      != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_get_date_mask: Vendor function unavailable");

     return (GDI_FAILURE);
}



/*
 ***************************** ora_get_date_now ****************************
 *
 *
 *
 *   get date based on now and the gmtime or localtime library routines.
 */
int
gdi_get_date_now (conn, gmt_or_local, lddate)
dbConn  *conn;
int     gmt_or_local;
char    *lddate;
{
        long    var1;
        struct  tm *var2;

	if ((conn == NULL) || (conn->vendor_conn == NULL))
	{
	     gdi_error_app (conn, GDI_NOCONNECT, 
			    "gdi_get_date_now: Database not connected");
	     return (GDI_FAILURE);
	}
     
        time (&var1);

        /* grab the time */
	if (gmt_or_local == GDI_GMT_TIME )
		var2 = gmtime (&var1);
	else
		var2 = localtime (&var1);

/* 
 * this should be changed to present the date in the format specified by the 
 * date mask in the vendor connector.  Vendor specific functions will probably
 * be called.   BGM - 5/21/93
 */
        sprintf (lddate, GDI_DEFAULT_DATE_MASK,
		 var2->tm_year,
		 var2->tm_mon+1,
		 var2->tm_mday,
		 var2->tm_hour,
		 var2->tm_min,
		 var2->tm_sec);

        return (GDI_SUCCESS);
}



/*
 ***************************** gdi_compute_jdate ****************************
 *
 *   compute jdate based on the time attribute and the gmtime
 *   library routine.
 */

long
gdi_compute_jdate(tyme)
double     tyme;
{
        long       epoch_time;
        long       jdate;
        struct tm  *time_ptr;
        int        day_of_year;

        epoch_time = (long) tyme;
        time_ptr = gmtime (&epoch_time);
        day_of_year = 1 + time_ptr->tm_yday;
        jdate = (time_ptr->tm_year + 1900) * 1000 + day_of_year;

        return(jdate);
}


