/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_dbobj.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for accessing the dbObj of the
 *	Generic Database Interface.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_dbobj.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_dbobj.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_obj_num_columns ()
 *
 * Get the number of columns in the dbObj
 *
 * Public
 */

int
gdi_obj_num_columns (dbobj)
dbObj	*dbobj;
{
     if (dbobj != NULL)
	  return (GDI_OBJ_NUM_COLUMNS (dbobj));
     else
	  return (0);
}



/*
 * gdi_obj_value ()
 *
 * Get a specified value from a specified tuple
 * Return NULL if not found.
 *
 * Public
 */

void *
gdi_obj_value (obj, tuple_num, col_num)
dbObj	*obj;
int	tuple_num;
int	col_num;
{
     dbTuple	*tuple;
     int	is_null;
     int	str_length;
     int	arr_length;
     
     if ((tuple = (dbTuple *) gdi_obj_tuple_retrieve (obj, tuple_num)) == NULL)
	  return (NULL);
     
     return (gdi_obj_get_data (obj, col_num, tuple, &is_null, &str_length, 
			       &arr_length));
}


/*
 * gdi_obj_col_num ()
 *
 * Get column number of a dbObj column given the column name.
 * Return -1 if not found.
 *
 * Public
 */

int
gdi_obj_col_num (dbobj, col_name)
dbObj	*dbobj;
char	*col_name;
{
     int	i;
     if (dbobj != NULL)
     {
	  for (i = 0; (i < GDI_OBJ_NUM_COLUMNS (dbobj)) && 
	       (GDI_OBJ_COL_DEF (dbobj, i) != NULL) &&
	       (strncmp (GDI_OBJ_COL_NAME (dbobj, i), col_name, GDI_MAX_NAME) != 0);
	       i++);
	  if ((i == GDI_OBJ_NUM_COLUMNS(dbobj)) || 
	      (GDI_OBJ_COL_DEF (dbobj, i) == NULL))
	       return (-1);
	  else
	       return (i);
     }	
     else
	  return (-1);
}


/*
 * gdi_obj_find_col_def ()
 *
 * Find the Col Def in a dbObj given the column name.   
 * Return NULL if not found.
 *
 * Public
 */

dbColDef *
gdi_obj_find_col_def (dbobj, col_name)
dbObj	*dbobj;
char	*col_name;
{
     int	i;
     
     i = gdi_obj_col_num (dbobj, col_name);
     if (i > 0)
	  return (GDI_OBJ_COL_DEF (dbobj, i));
     else
	  return (NULL);
}


/*
 * gdi_obj_find_value ()
 *
 * Find the value in a dbObj given the tuple number and the column name.   
 * Return NULL if not found.
 *
 * Public
 */

void *
gdi_obj_find_value (dbobj, tuple_num, col_name)
dbObj	*dbobj;
int	tuple_num;
char	*col_name;
{
     int	col_num;
     dbTuple	*tuple;
     int	is_null;
     int        str_length;
     int        arr_length;
     
     col_num = gdi_obj_col_num (dbobj, col_name);
     tuple = (dbTuple *) gdi_obj_tuple_retrieve (dbobj, tuple_num);

     if ((col_num >= 0) && (tuple != NULL))
	  return (gdi_obj_get_data (dbobj, col_num, tuple, &is_null, &str_length,
				    &arr_length));
     else
	  return (NULL);
}



dbObj *
gdi_obj_create (constr)
dbConstr	*constr;
{
     dbObj	*obj = NULL;
     
     if ((obj = UCALLOC (dbObj, 1)) != NULL)
     {
	  if (constr == NULL)
	       bcopy (&GDI_DEFAULT, GDI_OBJ_CONSTRUCTOR (obj), sizeof (dbConstr));
	  else
	       bcopy (constr, GDI_OBJ_CONSTRUCTOR (obj), sizeof (dbConstr));
     }
     
     return (obj);
}


dbObj*
gdi_obj_destroy (obj)
dbObj	*obj;
{
     dbColDef	**coldef;
     void	*tuple;
     int	i;
     
     if (obj != NULL)
     {
	  for (i = 0; i < GDI_OBJ_NUM_TUPLES (obj); i++)
	  {
	       tuple = gdi_obj_tuple_retrieve (obj, i);
	       gdi_obj_tuple_destroy (obj, tuple);
	  }
	  gdi_obj_container_destroy (obj);
	  for (coldef = GDI_OBJ_COL_DEFS (obj); (coldef != NULL) && (*coldef != NULL); 
	       coldef++)
	       UFREE (*coldef);
	  UFREE (GDI_OBJ_COL_DEFS (obj));
	  UFREE (GDI_OBJ_QUERY(obj));
	  UFREE (obj);
     }
     return (dbObj *)NULL;
}

