/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_error_init(), gdi_error_unix(),
 *	gdi_error_preface(), gdi_error_app(), gdi_warning_app(),
 *	gdi_error_get(), gdi_error_flags()
 *
 * DESCRIPTION
 *	This file contains functions for the generic database interface
 *	error handler.
 *
 * DIAGNOSTICS
 *	Functions which return pointers return NULL if an error
 *	occurrs.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	gdi_error.c (this file)
 *
 * NOTES
 *	Database-specific errors are set by the appropriate database
 *	vendor specific function.  Database errors are not set at
 *	the GDI level.
 * AUTHOR
 *	B. MacRitchie
 */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_error.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


dbConn	null_conn;

extern int	sys_nerr;			/* size of sys_errlist array */
extern char	*sys_errlist[];			/* system error messages */

#define MAX_SYS_ERR	256
#define NO_MSG	 	"No errno message"



/*
 * gdi_error_init ()
 *
 * Initializes errors for the connection
 *
 * Public
 */

int
gdi_error_init (conn, debug, threshold, reserved1, reserved2)
dbConn		*conn;
dbDebug 	debug;
dbErrLev	threshold;
int		reserved1;
int		reserved2;
{
     dbConn	*c;
     
     if (conn == NULL)
	  c = &null_conn;
     else
	  c = conn;

     c->debug = debug;
     if (threshold > GDI_FATAL)
	  c->threshold = GDI_FATAL;
     else
	  c->threshold = threshold;
     c->error_code = GDI_NOERROR;
     c->severity = GDI_NOERROR;
     c->status = GDI_SUCCESS;
     GDI_ERROR_MSG (c) [0] = '\0';
     
     return (GDI_SUCCESS);
}


/*
 * gdi_error_unix ()
 *
 * Set UNIX errors.
 *
 * Public
 */

int
gdi_error_unix (conn, errtext)
dbConn		*conn;
char		*errtext;
{
     dbConn	*c;
     int	errcode;
     char	errbuf [MAX_SYS_ERR + GDI_ERROR_SIZE + 1];
     
     if (conn == NULL)
	  c = &null_conn;
     else
	  c = conn;

     errcode = errno;
     strncpy (errbuf, errtext, GDI_ERROR_SIZE);
     errbuf [GDI_ERROR_SIZE] = '\0';
     strcat (errbuf, " -- ");

          /* 
	   * Make sure that this error is in the system error listing 
	   */
     if ((errcode < sys_nerr) && (errcode >= 0) )
	  strncat (errbuf, sys_errlist[errcode], strlen (sys_errlist[errcode]));
     else
	  strncat (errbuf, NO_MSG, strlen (NO_MSG));

     errbuf [MAX_SYS_ERR + GDI_ERROR_SIZE] = '\0';

     gdi_error_app (c, errcode, errbuf);

     return (c->status);
}


/*
 * gdi_error_preface ()
 *
 * Modify custom text by prefixing errtext to it.  Error codes
 * not changed.
 *
 * Public
 */

int
gdi_error_preface (conn, errtext)
dbConn		*conn;
char		*errtext;
{
     dbConn	*c;
     char	errbuf [GDI_ERROR_SIZE + 1];
     
     if (conn == NULL)
	  c = &null_conn;
     else
	  c = conn;

     strncpy (errbuf, errtext, GDI_ERROR_SIZE);
     errbuf [GDI_ERROR_SIZE] = '\0';
     strncat (errbuf, GDI_ERROR_MSG (c), GDI_ERROR_SIZE - strlen (errtext));
     errbuf [GDI_ERROR_SIZE] = '\0';

     strncpy (GDI_ERROR_MSG (c), errbuf, GDI_ERROR_SIZE);
     GDI_ERROR_MSG (c) [GDI_ERROR_SIZE] = '\0';

     if ((c->debug) && (c->error_code != GDI_NOERROR))
	  fprintf (stderr, "gdi_error: %s\n", errtext);

     return (c->status);
}


/*
 * gdi_error_app ()
 *
 * Set application errors.
 *
 * Public
 */

int
gdi_error_app (conn, errcode, errtext)
dbConn		*conn;
int		errcode;
char		*errtext;
{
     dbConn	*c;
     
     if (conn == NULL)
	  c = &null_conn;
     else
	  c = conn;

     c->status = GDI_SUCCESS;
     c->severity = GDI_NOERROR;
     GDI_ERROR_MSG (c) [0] = '\0';

     c->error_code = errcode;

     if (errtext != NULL)
     {
	  strncpy (c->custom_text, errtext, GDI_ERROR_SIZE);
	  GDI_ERROR_MSG (c) [GDI_ERROR_SIZE] = '\0';
     }

     if (c->error_code != GDI_NOERROR)
     {
	  c->status = GDI_FAILURE;
	  c->severity = GDI_FATAL;
     }
     
     if ((c->debug) && (c->error_code != GDI_NOERROR))
	  fprintf (stderr, "gdi_error: %05d: %s\n", c->error_code, GDI_ERROR_MSG (c));

     return (c->status);
}


/*
 * gdi_warning_app ()
 *
 * Set application warnings.
 *
 * Public
 */

int
gdi_warning_app (conn, errcode, errtext)
dbConn		*conn;
int		errcode;
char		*errtext;
{
     dbConn	*c;
     
     if (conn == NULL)
	  c = &null_conn;
     else
	  c = conn;

     c->status = GDI_SUCCESS;
     c->severity = GDI_NOERROR;
     GDI_ERROR_MSG (c) [0] = '\0';

     c->error_code = errcode;

     if (errtext != NULL)
     {
	  strncpy (c->custom_text, errtext, GDI_ERROR_SIZE);
	  GDI_ERROR_MSG (c) [GDI_ERROR_SIZE] = '\0';
     }

     if (c->error_code != GDI_NOERROR)
	  c->severity = GDI_WARNING;

     if (c->threshold <= c->severity)
	  c->status = GDI_FAILURE;
     
     if ((c->debug) && (c->error_code != GDI_NOERROR))
	  fprintf (stderr, "gdi_warning: %05d: %s\n", c->error_code, GDI_ERROR_MSG (c));

     return (c->status);
}


/*
 * gdi_error_get ()
 *
 * Get error settings for the connection.
 *
 * Public
 */

int
gdi_error_get (conn, errcode, errtext, maxtext, status, severity)
dbConn		*conn;
int		*errcode;
char		*errtext;
int		maxtext;
dbStatus	*status;
dbErrLev	*severity;
{
     dbConn	*c;
     char       *seperator = ":: ";
     
     if (conn == NULL)
	  c = &null_conn;
     else
	  c = conn;

     if (errcode != NULL)
	  *errcode = c->error_code;


     if ((errtext != NULL) && (maxtext >= 0))
     {
	  errtext [0] = '\0';
	  if ((c->error_code < GDI_BADDATA) && (c->vendor_conn != NULL) && 
	      ((GDI_FUNCS (c) != NULL) && (GDI_FUNCS (c)->error_get) != NULL))
	  {
	       GDI_FUNCS (c)->error_get (c, errtext, maxtext);
	       strncat (errtext, seperator, maxtext - strlen (errtext));
	       errtext [maxtext] = '\0';
	  }
	  strncat (errtext, GDI_ERROR_MSG (c), maxtext - strlen (errtext));
	  errtext [maxtext] = '\0';
     }

     if (status != NULL)
	  *status = c->status;
     
     if (severity != NULL)
	  *severity = c->severity;
     
     return (GDI_SUCCESS);
}



/*
 * gdi_error_flags ()
 *
 * Get error flag settings for the connection.
 *
 * Public
 */

int
gdi_error_flags (conn, debug, threshold)
dbConn		*conn;
dbDebug		*debug;
dbErrLev	*threshold;
{
     dbConn	*c;
     
     if (conn == NULL)
	  c = &null_conn;
     else
	  c = conn;

     if (debug != NULL)
	  *debug = c->debug;
     
     if (threshold != NULL)
	  *threshold = c->threshold;
     
     return (GDI_SUCCESS);
}

