/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_open(), gdi_close(), gdi_dead(), gdi_exit(),
 *	gdi_get_account(), gdi_get_database(), gdi_get_node()
 *
 * SYNOPSIS
 *
 *	#include "libgdi.h"
 *
 * DESCRIPTION
 *	This file contains the FORTRAN interfaces to the gdi functions 
 *      for managing the database connections
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 *      A global variable containing the single supported dbconn
 *      is maintained to support array inserts.
 * FILES
 *	f77_gdi_conn.c (this file)
 *	gdi_conn.c 
 *
 * SEE ALSO
 *
 *	libgdi.h
 *      f77_db_conn.h
 *      f77_db_conn.c
 *
 * AUTHOR
 *	H. Turner
 */

#include "utilities.h"
#include "libgdi.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_conn.c	16.3 8/17/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

/* dbConn F77_conn;
 * int    F77_conn_active = 0;
 */


static	char *vend = (char *)NULL;
static 	char *acnt = (char *)NULL;
static 	char *passwd = (char *)NULL;
static	char *db = (char *)NULL;
static 	char *serv = (char *)NULL;
static	char *app = (char *)NULL;

static void free_open_strings()
{
	UFREE(vend);
	UFREE(acnt);
	UFREE(passwd);
	UFREE(db);
	UFREE(serv);
	UFREE(app);
}

int gdi_open_(vendor, account, password, database, server, appname,
	      lvendor, laccount, lpassword, ldatabase, lserver, lappname)
char	*vendor;
char	*account;
char	*password;
char	*database;
char	*server;
char	*appname;
int     lvendor;
int 	laccount;
int 	lpassword;
int 	ldatabase;
int 	lserver;
int	lappname;
{

	int i;
    
	char *fname = "gdi_open_ :";
	
	dbConn *conn = (dbConn *)NULL;
	
	vend = (char *)NULL;
	acnt = (char *)NULL;
	passwd = (char *)NULL;
	db = (char *)NULL;
	serv = (char *)NULL;
	app = (char *)NULL;

#if 0
	if (F77_conn_active)
	{
		gdi_error_app((dbConn*)NULL,GDI_BADDATA,
			      "gdi_open_: connection already open.  Connection refused.");
		return gdicommon_.gdi_noconn;
	}
#endif	
	if (lvendor > 0)
	{
		if ((vend = UALLOC(char, lvendor+1)) == (char *)NULL)
		{
			gdi_error_unix(conn, fname);
			free_open_strings();
			return (gdicommon_.gdi_noconn);
		}
		(void) gdi_for_to_cstr(vend, lvendor+1, vendor, lvendor);
	}		

	if (laccount > 0)
	{
		if ((acnt = UALLOC(char, laccount+1)) == (char *)NULL)
		{
			gdi_error_unix(conn, fname);
			free_open_strings();
			return (gdicommon_.gdi_noconn);
		}
		(void) gdi_for_to_cstr(acnt, laccount+1, account, laccount);
	}
	
	if (lpassword > 0)
	{
		if ((passwd = UALLOC(char, lpassword+1)) == (char *)NULL)
		{
			gdi_error_unix(conn, fname);
			free_open_strings();
			return (gdicommon_.gdi_noconn);
		}
		(void) gdi_for_to_cstr(passwd, lpassword+1, password, lpassword);
	}
	
	if (ldatabase > 0)
	{
		if ((db = UALLOC(char, ldatabase+1)) == (char *)NULL)
		{
			gdi_error_unix(conn, fname);
			free_open_strings();
			return (gdicommon_.gdi_noconn);
		}
		(void) gdi_for_to_cstr(db, ldatabase+1, database, ldatabase);
	}
	
	if (lserver > 0)
	{
		if ((serv = UALLOC(char, lserver+1)) == (char *)NULL)
		{
			gdi_error_unix(conn, fname);
			free_open_strings();
			return (gdicommon_.gdi_noconn);
		}
		(void) gdi_for_to_cstr(serv, lserver+1, server, lserver);
	}
	
	if (lappname > 0)
	{
		if ((app = UALLOC(char, lappname+1)) == (char *)NULL)
		{
			gdi_error_unix(conn, fname);
			free_open_strings();
			return (gdicommon_.gdi_noconn);
		}
		(void) gdi_for_to_cstr(app, lappname+1, appname, lappname);
	}
	
	
	conn = gdi_open(vend, 
                        acnt, 
                        passwd, 
                        db, 
                        serv, 
                        app);

	free_open_strings();

	if (conn == (dbConn *)NULL)
	{
		return gdicommon_.gdi_noconn;
	}

	/* add the connection to the connection table so
         * we can retrieve it by "id" later
         */
	i = add_db_conn(conn);

#if 0
	/* load the global connection for FORTRAN so we can 
         * use it to do the array_insert routines.
         */
	bcopy (conn, &F77_conn, sizeof(dbConn));
	F77_conn_active = 1;
#endif		
	return (i);
}


int gdi_get_node_(conn, node, len)
int *conn;
char *node;
int len;
{
	char *temp;
	int i;
	dbConn *connp = get_db_conn(*conn);
	
	if ((temp = UALLOC(char, len+1)) == (char *)NULL)
	{
		gdi_error_unix(connp, "gdi_get_node_: ");
		UFREE(temp);
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(node, len+1, temp, len);

	i = gdi_get_node(connp,
			 temp,
			 len);
	UFREE(temp);

	return (i);
}


int gdi_close_(conn)
int *conn;
{
/*	F77_conn_active = 0;
 */
	return (gdi_close(get_db_conn(*conn)));
}

int gdi_exit_()
{
	return (gdi_exit());
}

int gdi_dead_(conn)
int *conn;
{
	return (gdi_dead(get_db_conn(*conn)));
}

int gdi_get_database_(conn, db, len, db_len)
int *conn;
char *db;
int *len;
int db_len;
{
	/* Not implemented yet */
	return (GDI_FAILURE);
}

int gdi_get_account_(conn, account, len, acnt_len)
int *conn;
char *account;
int *len;
int acnt_len;
{
	/* Not implemented yet */
	return (GDI_FAILURE);
}



