/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_f77_map.c
 *
 * SYNOPSIS
 *
 *	#include "libgdi.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains global data structures and its support functions 
 *      for the FORTRAN 77 interface to the gdi.
 *      
 *      FORTRAN will reference its database connections and its mapping
 *      references as integers.  Since the gdi is expecting one of its
 *      structures as defined in libgdi.h, we must translate between
 *      what FORTRAN expects and what C expects.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_f77_map.c
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */

#include <stdio.h>
#include "libgdi.h"
#include "utilities.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_map.c	16.2 8/11/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif



#define F77MAPINC 10



/* global data strutures */

static int init = 0;



/* mapping array */
static Col_Name_Addr **f77_mapping = (Col_Name_Addr **)NULL;
static int *slot_active = (int *)NULL;
static int mapping_size = 0;
static int in_use = 0;



/* local procedures / functions */
static int i;

static
Col_Name_Addr *make_elem(conn)
dbConn *conn;
{
	Col_Name_Addr *elem;

	elem = UALLOC(Col_Name_Addr,1);
	if (elem == (Col_Name_Addr *)NULL)
	{
		(void) gdi_error_unix(conn, "make_elem: ");
		return ((Col_Name_Addr *)NULL);
	}
	elem->column_name = (char *)NULL;
        elem->row_count = 0;
	elem->addr = (void *)NULL;
	elem->type = -1;	
	elem->elem_size = -1;
	elem->array_size = -1;
	elem->string_len = -1;
	elem->next = (Col_Name_Addr *)NULL;
	
	return elem;
}

/* C callable routines */

/* retrieve an entry for the gdi constructor */

Col_Name_Addr *
retrieve_map(map)
int map;
{
	map--;
	
	if ((map > mapping_size) ||
	    (!slot_active[map]))
	{
		return ((Col_Name_Addr *)NULL);
	}
	return (f77_mapping[map]);
}

/* perform run-time initializations */
int gdi_initialize_mapping()
{
	char *fname = "gdi_initialize_mapping: ";
	
	init = 1;
	/*
	 * first time through, allocate the arrays that hold
	 * the mapping pointers.
	 */
	f77_mapping = UALLOC(Col_Name_Addr*,F77MAPINC);
	if (f77_mapping == (Col_Name_Addr **)NULL)
	{
/*		(void) gdi_error_unix(conn, fname);
 */
		fprintf(stderr,"malloc failed during gdi init\n");
		
		return GDI_FAILURE;
	}
	slot_active = UALLOC(int,F77MAPINC);
	if (slot_active == (int *)NULL)
	{
/*		(void) gdi_error_unix(conn, fname);
 */
		fprintf(stderr,"malloc failed during gdi init\n");
		
		return GDI_FAILURE;
	}		
	mapping_size = F77MAPINC;

	for (i=0; i<mapping_size; i++)
	{
		f77_mapping[i] = (Col_Name_Addr *)NULL;
		slot_active[i] = 0;
	}

	return gdicommon_.gdi_success;
	
}


/* FORTRAN interface */
/* begin a mapping reference.  */
int gdi_open_map_ (conn)
int  *conn;
{
	char *routine = "gdi_open_map: ";
	int j;
	dbConn *c;
	
	
	if (!init)
	{
		return -1;
	}
	c = get_db_conn(*conn);
	
	j = 0;
	if (in_use == mapping_size)
	{
		/* allocate more space */
		f77_mapping = UREALLOC(f77_mapping,Col_Name_Addr*,F77MAPINC);
		if (f77_mapping == (Col_Name_Addr **)NULL)
		{
			(void) gdi_error_unix( c, routine);
			return gdicommon_.gdi_nomap;
		}
		j = mapping_size;
		mapping_size += F77MAPINC;
		for (i = j; i<mapping_size; i++)
		{
			f77_mapping[i] = (Col_Name_Addr *)NULL;
			slot_active[i] = 0;
		}
	}
	
	if (j == 0)
	{
		/* search for an available slot */
		for (i=0; i<mapping_size; i++)
		{
			if (!slot_active[i])
			{
				slot_active[i] = 1;
				return (++i);
			}
		}
		/* should never get here.
		 * If you get here, it means that the mapping array is 
		 * a mess.
		 */
		(void) gdi_error_app( c, GDI_BADDATA, 
				     "gdi_open_map: mapping data structure corrupted");
		return gdicommon_.gdi_nomap;
	}

	/*else*/
	/* use the slot named by j */
	if (slot_active[j])
	{
		(void) gdi_error_app( c, GDI_BADDATA, 
				     "gdi_open_map: Can't find empty map ");
		return gdicommon_.gdi_nomap;
	}
	slot_active[j] = 1;	
	return (++j);
}

void gdi_close_map_(conn, map)
int *conn;
int *map;
{
	return;
}

void gdi_destroy_map_(conn, map_id)
int  *conn;
int *map_id;
{

	Col_Name_Addr *elem;
	Col_Name_Addr *nxt;
        int map = *map_id;
        
	
        map--;
	if ((!init) ||
	    (map >= mapping_size) ||
	    (map < 0))
	{
		return;
	}

	nxt = f77_mapping[map];

	while (nxt !=  (Col_Name_Addr *)NULL)
	{
		elem = nxt;
		nxt = nxt->next;
		UFREE(elem->column_name);
		UFREE(elem);
	}
	f77_mapping[map] = (Col_Name_Addr *)NULL;
	slot_active[map] = 0;
}

int gdi_add_map_field_ (conn,
			map_id,	
			column_name, 
			data_addr, 
			data_type, 
			string_len,
			array_len,
			column_len)
int  *conn;
int  *map_id;
char *column_name;
void *data_addr;
int  *data_type;
int  *array_len;
int  *string_len;
int  column_len;

{
	dbConn *c;
	Col_Name_Addr *elem;
	Col_Name_Addr *nxt;
        int map = *map_id;
        
        map--;
        
	if ((!init) ||
	    (map >= mapping_size) ||
	    (map < 0))
	{
		return -1;
	}
	c = get_db_conn(*conn);
	elem = make_elem(c);
	if (column_len > 0)
	{
		if ((elem->column_name = UALLOC(char, column_len+1)) == (char *)NULL)
		{
			gdi_error_unix(c, "gdi_add_map_field_");
			return (GDI_FAILURE);
		}
		(void) gdi_for_to_cstr(elem->column_name, 
				       column_len+1, 
				       column_name, 
				       column_len);
	}		
		
        elem->row_count = 0;
	elem->addr = data_addr;
	elem->type = *data_type;
	elem->string_len = *string_len; 
	elem->array_size = *array_len;
	elem->next = (Col_Name_Addr *)NULL;

	switch (*data_type) 	
	{

	case GDI_UNDEFINED:           
		elem->elem_size = 0;
		break;

	case GDI_FLOAT:               
		elem->elem_size = sizeof(float);
		break;

	case GDI_SHORT:               
		elem->elem_size = sizeof(short);
		break;
		
	case GDI_LONG:                
	case GDI_INTEGER:                
		elem->elem_size = sizeof(int);
		break;
		
	case GDI_DOUBLE:
		elem->elem_size = sizeof(double);
		break;
		
	case GDI_STRING:
		elem->elem_size = sizeof(char *);
		break;
		
	case GDI_USER_DEFINED:
		elem->elem_size = 0;		
		break;
		
	case GDI_CHAR:
		elem->elem_size = sizeof(char);		
		break;

	default:
		/* not one of the defined data sizes */
		UFREE(elem->column_name);
		UFREE(elem);

		gdi_error_app(c, GDI_BADDATA, 
			      "gdi_add_map_field : unrecogized FORTRAN datatype");
		
		return GDI_FAILURE;
		break;
	}
	
	nxt = f77_mapping[map];

	if (nxt == (Col_Name_Addr *)NULL)
	{
		f77_mapping[map] = elem;
	}
	else
	{	
		for (;nxt->next != (Col_Name_Addr *)NULL; nxt = nxt->next)
		{
		}
		nxt->next = elem;

	}
	return GDI_SUCCESS;
}	
