/* Copyright 1992 Science Applications International Corporation
 * 
 * This software may not be used, copied, modified, or distributed without
 * the express written permission of Science Applications International 
 * Corporation (SAIC).  SAIC makes no warranty of any kind with regard
 * to this software, including, but not limited to, the implied warranties
 * of fitness for a particular purpose.
 * 
 * Thomas C. Bache, Vice President, SAIC, San Diego CA (bache@esosun.css.gov)
 */

/*
 * NAME
 *	gdi_gobj.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *	#include "gdi_doi.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for creating the Gobj tuple of the
 *	Generic Database Interface.
 *
 * 	The functions are duplicates of the original default tuple 
 * 	constructor functions, namely with tuple replaced by gobj.  This 
 *	set of functions currently creates an array of gobjs.  They will 
 *	have to be changed to create a hash TABLE of gobjs.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_gobj.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *	gdi_doi.h
 *
 * AUTHOR
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"
#include "gdi_doi.h"

#ifndef	lint
static char	SccsId[] = "@(#)gdi_gobj.c	13.1 5/17/93	SAIC";
#endif


/*
 * prototypes for GObj Constructor
 */

Proto (static int, gdi_gobj_add, 
       (
	GObj ***gobj_array,		/* gobj array pointer */
	GObj *gobj,			/* gobj               */
	void **args                     /* pointer to constructor arguments */
	));

Proto (static GObj *, gdi_gobj_retrieve, 
       (
	GObj **gobj, 			/* gobj pointer */
	int  gobj_num,                  /* gobj number  */
	void **args                     /* pointer to constructor arguments */
	));

Proto (static int, gdi_gobj_destroy, 
       (
	GObj *gobj,			/* gobj */
	void **args                     /* pointer to constructor arguments */
	));

Proto (static GObj *, gdi_gobj_create, 
       (
	dbColDef **col_def, 		/* column defintion */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static int, gdi_gobj_fill_value, 
       (
	int      col_num, 		/* column number  */
	dbColDef *col_def, 		/* column definition */
	GObj     *gobj, 		/* gobj          */
	void     *dbdata, 		/* database data  */
	int	 is_null, 		/* "is null" flag */
	int      length,		/* data length    */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static void *, gdi_gobj_get_value, 
       (
	int      col_num, 		/* column number  */
	dbColDef *col_def, 		/* column definition */
	GObj     *gobj, 		/* gobj          */
	int      *is_null, 		/* "is null" flag */
	int      *length,		/* data length    */
	void     **args                 /* pointer to constructor arguments */
	));



#ifdef __STDC__
dbConstr	GDI_GOBJ = 
{
     (int    (*) ())		                NULL,
     (void * (*) ())			        NULL,
     (int    (*) (void **, void *, void **))	gdi_gobj_add,
     (void * (*) (void *, int, void **))	gdi_gobj_retrieve,
     (int    (*) (void *, void **))		gdi_gobj_destroy,
     (void * (*) (dbColDef **, void **))        gdi_gobj_create,
     (int    (*) (int, dbColDef *, void *, void *, int, int, void **))
	                                        gdi_gobj_fill_value,
     (void * (*) (int, dbColDef *, void *, int *, int *, void **))
	                                        gdi_gobj_get_value,
     (int    (*) ())		                NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DEFAULT_COERCE,
};
#else
dbConstr	GDI_GOBJ = 
{
     (int    (*) ())	NULL,
     (void * (*) ())	NULL,
     (int    (*) ())	gdi_gobj_add,
     (void * (*) ())	gdi_gobj_retrieve,
     (int    (*) ())	gdi_gobj_destroy,
     (void * (*) ())	gdi_gobj_create,
     (int    (*) ())	gdi_gobj_fill_value,
     (void * (*) ()) 	gdi_gobj_get_value,
     (int    (*) ()) 	NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DEFAULT_COERCE,
};
#endif



/*
 * gdi_gobj_add ()
 *
 * Add a gobj to the gobj array.
 *
 * Private
 */
static int
gdi_gobj_add (garray, gobj, args)
GObj		***garray;
GObj		*gobj;
void            **args;
{
     int	i;
     GObj	**g;
     char	*routine = "gdi_gobj_add: ";
     
     if (garray == NULL)
	  return (GDI_FAILURE);
     
     for (i = 0; (*garray != NULL) && (*garray)[i] != NULL; i++);

     if ((g = UREALLOC ((*garray), GObj *, i + 2)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (GDI_FAILURE);
     }
     
     *garray = g;
     
     (*garray) [i] = gobj;
     (*garray) [i+1] = NULL;

     return (GDI_SUCCESS);
}


/*
 * gdi_gobj_retrieve ()
 *
 * Retrieve a gobj from the gobj array.
 *
 * Private
 */
static GObj *
gdi_gobj_retrieve (garray, gobj_num, args)
GObj		**garray;
int		gobj_num;
void            **args;
{
     if (garray == NULL)
	  return (NULL);
     
     return (garray [gobj_num]);
}


/*
 * gdi_gobj_destroy ()
 *
 * Destroy a gobj.
 *
 * Private
 */
static int
gdi_gobj_destroy (gobj, args)
GObj		*gobj;
void            **args;
{
     int	n;
     
     if (gobj == NULL)
	  return (GDI_SUCCESS);
     
     for (n = 0; n < GDI_GOBJ_NUM_COLUMNS(gobj); n++)
	  if (GDI_GOBJ_CTYPE (gobj, n) == GDI_STRING)
	       UFREE (GDI_GOBJ_VALUE(gobj, n)->string);

     UFREE (gobj);

     return (GDI_SUCCESS);
}     
	  
     
/*
 * gdi_gobj_create ()
 *
 * Create a new gobj.
 *
 * Private
 */
static GObj *
gdi_gobj_create (coldef, args)
dbColDef	**coldef;
void		**args;
{
     int	i;
     int	n;
     GObj	*gobj;
     char	*routine = "gdi_obj_create: ";
     
     if (coldef == NULL)
	  return (NULL);
     
     if ((gobj = UCALLOC (GObj, 1)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (NULL);
     }
     
     for (n = 0; coldef[n] != NULL; n++);
     if ((gobj->col = UCALLOC (gobjCol, n)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  UFREE (gobj);
	  return (NULL);
     }
     
     GDI_GOBJ_NUM_COLUMNS(gobj) = n;

     for (i = 0; i < n; i++)
     {
	  strncpy (GDI_GOBJ_NAME (gobj, i), coldef[i]->name, GDI_MAX_NAME);
	  strncpy (GDI_GOBJ_DBTYPE_S (gobj, i), coldef[i]->dbtype_s, GDI_MAX_TYPE);
	  GDI_GOBJ_CTYPE (gobj, i) = coldef[i]->ctype;
     }
     return (gobj);
}


/*
 * gdi_gobj_fill_value ()
 *
 * Fill a gobj column value with database data.  Length for a string
 * may or may not include a NULL terminator.
 *
 * Private
 */
static int
gdi_gobj_fill_value (col_num, col_def, gobj, dbdata, is_null, length, args)
int		col_num;
dbColDef	*col_def;
GObj		*gobj;
void		*dbdata;
int		is_null;
int		length;
void            **args;
{
     char	*routine = "gdi_gobj_fill_value: ";
     
     if ((col_num < 0) || (gobj == NULL))
	  return (GDI_SUCCESS);
     
     GDI_GOBJ_IS_NULL (gobj, col_num) = is_null;
     GDI_GOBJ_LENGTH (gobj, col_num) = length;
     switch (GDI_GOBJ_CTYPE (gobj, col_num))
     {
     case GDI_CHAR:
	  GDI_GOBJ_VALUE (gobj, col_num)->char_val = * ((char *) dbdata);
	  break;
     case GDI_INTEGER:
	  GDI_GOBJ_VALUE (gobj, col_num)->long_val = * ((int *) dbdata);
	  break;
     case GDI_LONG:
	  GDI_GOBJ_VALUE (gobj, col_num)->long_val = * ((long *) dbdata);
	  break;
     case GDI_DOUBLE:
	  GDI_GOBJ_VALUE (gobj, col_num)->double_val = * ((double *) dbdata);
	  break;
     case GDI_FLOAT:
	  GDI_GOBJ_VALUE (gobj, col_num)->double_val = * ((float *) dbdata);
	  break;
     case GDI_STRING:
	  if ((GDI_GOBJ_VALUE (gobj, col_num)->string = UCALLOC (char,
							     length +
								 1))
	      == NULL)
	  {
	       gdi_error_unix (NULL, routine);
	       return (GDI_FAILURE);
	  }
	  
	  bcopy ((char *) dbdata , GDI_GOBJ_VALUE (gobj, col_num)->string, 
		 length * sizeof (char));
	  GDI_GOBJ_VALUE (gobj, col_num)->string [length] = '\0';
	  break;
     default:
	  break;
     }
     return (GDI_SUCCESS);
}


/*
 * gdi_gobj_get_value ()
 *
 * Return a the gobj column value.  Also fill in is_null and length.
 *
 * Private
 */
static void *
gdi_gobj_get_value (col_num, col_def, gobj, is_null, length, args)
int		col_num;
dbColDef	*col_def;
GObj		*gobj;
int		*is_null;
int		*length;
void            **args;
{
     void	*dbdata;
     
     if ((col_num < 0) || (gobj == NULL))
	  return (NULL);


     if (is_null != NULL)
	  *is_null = GDI_GOBJ_IS_NULL (gobj, col_num);
     
     if (length != NULL)
	  *length = GDI_GOBJ_LENGTH (gobj, col_num);
     
     if (col_def->ctype == GDI_STRING)
	  dbdata = GDI_GOBJ_VALUE (gobj, col_num)->string;
     else
	  dbdata = GDI_GOBJ_VALUE (gobj, col_num);
     
     return (dbdata);
}

