/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_print_conn(), gdi_print_dbobj(), gdi_print_coldefs(),
 *	gdi_print_tuple_header(), gdi_print_tuples()
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *	int
 *	gdi_print_conn (conn)
 *	dbConn	*conn;
 *
 *	int
 *	gdi_print_dbobj ( gdi_obj )
 *	dbObj *gdi_obj;		dbobj to print out
 *
 *	int
 *	gdi_print_coldefs ( gdi_obj )
 *	dbObj *gdi_obj;		dbobj to print out
 *
 *	int
 *	gdi_print_tuple_header ( gdi_obj, format, header)
 *	dbObj *gdi_obj;		dbobj to print out
 *	int	format;		GDI_FIXED_SPACE or GDI_DELIMITED
 *	int	header;		TRUE to print headings.  FALSE for data only
 *
 *	int
 *	gdi_print_tuples ( gdi_obj, format, header)
 *	dbObj *gdi_obj;		dbobj to print out
 *	int	format;		GDI_FIXED_SPACE or GDI_DELIMITED
 *	int	header;		TRUE to print headings.  FALSE for data only
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for printing the contents of a
 *	database connector (dbConn), a database object (dbObj),
 *	column definitions and tuple data to stdout.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_print.c (this file)
 *
 * SEE ALSO
 *	gdi_print_conn(3), gdi_print_coldefs(3), gdi_print_dbobj(3),
 *	gdi_print_tuples(3)
 *
 * AUTHOR
 *	B. MacRitchie
 *	M. Mortell	Jan. 93		Added gdi_print_dbobj.c
 *					      gdi_print_cold[-1zef.c
 *					      gdi_print_tuples.c
 */


#include <stdio.h>
#include "utilities.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_print.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_print_conn ()
 *
 * Print the contents of a database connection to stdout.
 *
 * Public
 */

int
gdi_print_conn (conn)
dbConn	*conn;
{
     int    ret;
     char   *fname = "gdi_print_conn: ";
     
     if (conn == NULL)
     {
	  gdi_warning_app (conn, GDI_NOCONNECT,
			   "gdi_close: Not connected to database");
	  return (GDI_FAILURE);
     }

     fprintf (stdout, "DATABASE CONNECTOR = 0x%p\n", conn);
     fprintf (stdout, "   vendor_id = %d ", conn->vendor_id);
     switch(conn->vendor_id)
	{
	case GDI_ORACLE:
		fprintf (stdout, "(%s)\n", GDI_ORACLE_S);
		break;
	case GDI_POSTGRES:
		fprintf (stdout, "(%s)\n", GDI_POSTGRES_S);
		break;
	case GDI_SYBASE:
		fprintf (stdout, "(%s)\n", GDI_SYBASE_S);
		break;
	case GDI_INGRES:
		fprintf (stdout, "(%s)\n", GDI_INGRES_S);
		break;
	case GDI_UNKNOWN:
		fprintf (stdout, "(%s)\n", GDI_UNKNOWN_S);
		break;
	}

     fprintf (stdout, "   account = '%s', ", conn->account);
     fprintf (stdout, "   database = '%s', ", conn->database);
     fprintf (stdout, "   node = '%s', \n", conn->node);
     fprintf (stdout, "   os_pid = %d,\n", conn->os_pid);

     fprintf (stdout, "   query language = ");

     switch(conn->lang)
	{
	case GDI_SQL:
     		fprintf (stdout, "sql\n");
		break;
	case GDI_POSTQUEL:
     		fprintf (stdout, "postquel\n");
		break;
	}

     fprintf (stdout, "   error_handler () = 0x%p, ", conn->error_handler);
     fprintf (stdout, "   intr_handler () = 0x%p\n", conn->intr_handler);
     fprintf (stdout, "   status = %d\n", conn->status);
     fprintf (stdout, "   severity = %d, ", conn->severity);
     fprintf (stdout, "   error_code = %d\n", conn->error_code);
     fprintf (stdout, "   custom_text = '%s'\n", conn->custom_text);
     fprintf (stdout, "   debug = %d ", conn->debug);
     switch(conn->debug)
	{
	case GDI_DEBUG_OFF:
	     fprintf (stdout, " (GDI_DEBUG_OFF), ");
	     break;
	case GDI_DEBUG_ON:
	     fprintf (stdout, " (GDI_DEBUG_ON), ");
	     break;
	case GDI_DEBUG_VERBOSE:
	     fprintf (stdout, " (GDI_DEBUG_VERBOSE), ");
	     break;
	}

     fprintf (stdout, "   threshold = %d\n", conn->threshold);

     fprintf (stdout, "   prev_conn = 0x%p, ", conn->prev_conn);
     fprintf (stdout, "   next_conn = 0x%p, \n", conn->next_conn);
     fflush (stdout);
     
     if (conn ->vendor_conn == NULL)
     {
	  fprintf (stdout, "   DATABASE VENDOR CONNECTOR = 0x%p\n", conn->vendor_conn);
	  fflush (stdout);
	  return (GDI_SUCCESS);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->print_conn) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->print_conn (conn)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     return (GDI_SUCCESS);
}

/* 
 * gdi_print_dbobj - no data or column defs printed here.
 *
 */

int
gdi_print_dbobj ( gdi_obj )
dbObj *gdi_obj;		/*	dbobj to print out	*/
{
	
	dbObj* this_dbobj;
	char *underline = "--------------------------------------";
	char more_rows[5];
	
	/*
         * Verify the input parameters.
         */

	if( gdi_obj == NULL )
        {
                gdi_error_app( NULL, GDI_BADDATA,
                              "gdi_print_tuples: NULL input dbObj: ");
                return (GDI_FAILURE);
        }

	this_dbobj=gdi_obj;

	fprintf (stdout,
		 "\n%-*s %-*s %-*s %-*s %-*s\n",
		 strlen("Affected Rows"), "Affected Rows", 
		 strlen("Tuples "), "Tuples ", 
		 strlen("Columns"), "Columns",
		 strlen("Status"), "Status",
		 strlen("More Rows"), "More Rows");
	
	fprintf (stdout,
		 "%.*s %.*s %.*s %.*s %.*s\n",
		 strlen("Affected Rows"), underline, 
		 strlen("Tuples "), underline, strlen("Columns"), underline,
		 strlen("Status"), underline,
		 strlen("More Rows"), underline);

	if (GDI_OBJ_MORE_ROWS(this_dbobj) == TRUE)
		strcpy(more_rows, "TRUE");
	else
		more_rows[0] = '\0';
	
	fprintf (stdout,
		 "%*d %*d %*d %*d %-*.*s\n",
		 strlen("Affected Rows"), GDI_OBJ_ROWS_AFFECTED (this_dbobj),
		 strlen("Tuples "), GDI_OBJ_NUM_TUPLES (this_dbobj),
		 strlen("Columns"),  gdi_obj_num_columns(this_dbobj),
		 strlen("Status"), GDI_OBJ_STATUS(this_dbobj),
		 strlen("More Rows"), strlen("More Rows"), more_rows);
	
	fprintf (stdout, "%-*s\n", strlen("Query"), "Query");
	fprintf (stdout, "%.*s\n", strlen("Query"), underline);

	if ( GDI_OBJ_QUERY (this_dbobj) == NULL )
		fprintf ( stdout, "\n");
	else
		fprintf (stdout, "%-s\n", GDI_OBJ_QUERY (this_dbobj));
	
	return (GDI_SUCCESS);
	
}


/* 
 * gdi_print_coldefs - the column definitions
 *
 */

int
gdi_print_coldefs ( gdi_obj )
dbObj *gdi_obj;		/*	dbobj to print out	*/
{

	dbObj* this_dbobj;
	int i;
	char *underline = "----------------------------------------------";
	/*
         * Verify the input parameters.
         */

	if( gdi_obj == NULL )
        {
                gdi_error_app(NULL, GDI_BADDATA,
                              "gdi_print_tuples: NULL input dbObj: ");
                return (GDI_FAILURE);
        }

	this_dbobj=gdi_obj;
	fprintf (stdout,
		"\n%-*s %-*s %-*s %-*s %-*s %-*s %-*s %-*s %-*s\n",
		GDI_MAX_NAME/2, "Name", 
		strlen("Null?"), "Null?",
		strlen("Ctype"), "Ctype",
		strlen("StrSize"), "StrSize",
		strlen("ArraySize"), "ArraySize",
		strlen("Prec"), "Prec",
		strlen("Scale"), "Scale",
		strlen("Dbtype"), "Dbtype",
		strlen("DbtypeStr"), "DbtypeStr");
	fprintf (stdout,
		 "%.*s %.*s %.*s %.*s %.*s %.*s %.*s %.*s %.*s\n",
		GDI_MAX_NAME/2, underline, 
		strlen("Null?"), underline,
		strlen("Ctype"), underline,
		strlen("StrSize"), underline,
		strlen("ArraySize"), underline,
		strlen("Prec"), underline,
		strlen("Scale"), underline,
		strlen("Dbtype"), underline,
		strlen("DbtypeStr"), underline);

	for(i=0; i < gdi_obj_num_columns(this_dbobj); i++)
	{
		fprintf (stdout, "%-*.*s %*d %*d %*d %*d %*d %*d %*d %-*s\n",
			GDI_MAX_NAME/2, GDI_MAX_NAME/2,
			GDI_OBJ_COL_NAME (this_dbobj, i),
			strlen("Null?"),GDI_OBJ_COL_ALLOW_NULL (this_dbobj,i),
			strlen("Ctype"), GDI_OBJ_COL_CTYPE (this_dbobj, i),
			strlen("StrSize"), GDI_OBJ_COL_LENGTH (this_dbobj, i),
			strlen("ArraySize"), GDI_OBJ_COL_MAX_ARRLEN (this_dbobj, i),
			strlen("Prec"), GDI_OBJ_COL_PRECISION (this_dbobj, i),
			strlen("Scale"), GDI_OBJ_COL_SCALE (this_dbobj, i),
			strlen("Dbtype"), GDI_OBJ_COL_DBTYPE (this_dbobj, i),
			strlen("DbtypeStr"), 
			GDI_OBJ_COL_DBTYPE_S (this_dbobj, i));

	}
	fprintf(stdout, "\n");

	return ( GDI_SUCCESS );
}
/*
 * gdi_print_tuple_header
 *
 */
#define DBL_WIDTH  16
#define DBL_PREC   3
#define LONG_WIDTH 10

int
gdi_print_tuple_header (gdi_obj, format)
dbObj    *gdi_obj;		/* dbobj to print out	            */
dbFormat format;		/* GDI_FIXED_SPACE or GDI_DELIMITED */
{
	
     int  j;
     int  ncols;
     int  clen;
     int  underline_len;
     char *underline = "-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------";
     
     /*
      * Verify the input parameters.
      */
     
     if (gdi_obj == NULL )
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_print_tuples: NULL input dbObj");
	  return (GDI_FAILURE);
     }
     
     ncols = gdi_obj_num_columns (gdi_obj);

     if (format == GDI_FIXED_SPACE )
     {
	  for (j = 0; j < ncols; j++)
	  {
	       switch (GDI_OBJ_COL_CTYPE (gdi_obj, j))
	       {
	       case GDI_STRING:
		    clen = MAX (GDI_OBJ_COL_LENGTH (gdi_obj, j), 
				strlen (GDI_OBJ_COL_NAME (gdi_obj, j)));
		    fprintf (stdout, "%-*.*s ", clen, clen, 
			     GDI_OBJ_COL_NAME (gdi_obj, j));
		    break;
	       case GDI_CHAR:
		    clen = strlen (GDI_OBJ_COL_NAME(gdi_obj, j));
		    fprintf (stdout, "%-*.*s ", clen, clen, 
			     GDI_OBJ_COL_NAME (gdi_obj, j));
		    break;
	       case GDI_DOUBLE:
	       case GDI_FLOAT:
		    clen = MAX (DBL_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    fprintf (stdout, "%*.*s ",  clen, clen,
			     GDI_OBJ_COL_NAME (gdi_obj, j));
		    break;
	       case GDI_LONG:
	       case GDI_INTEGER:
	       case GDI_SHORT:
		    clen = MAX (LONG_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    fprintf (stdout, "%*.*s ", clen, clen, 
			     GDI_OBJ_COL_NAME (gdi_obj, j));
		    break;
	       default:
		    fprintf (stdout, "%s ", GDI_OBJ_COL_NAME (gdi_obj, j));
	       }
	  }
	  fprintf (stdout, "\n");

	  /*
	   * Underline column headings
	   * --------- ------ --------
	   */
	  for (j = 0; j < ncols; j++)
	  {
	       switch (GDI_OBJ_COL_CTYPE (gdi_obj, j))
	       {
	       case GDI_STRING:
		    clen = MAX (GDI_OBJ_COL_LENGTH (gdi_obj, j), 
				strlen (GDI_OBJ_COL_NAME (gdi_obj, j)));
		    fprintf (stdout, "%.*s ", clen, underline);
		    break;
	       case GDI_CHAR:
		    clen = strlen (GDI_OBJ_COL_NAME(gdi_obj, j));
		    fprintf (stdout, "%.*s ", clen, underline);
		    break;
	       case GDI_DOUBLE:
	       case GDI_FLOAT:
		    clen = MAX (DBL_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    fprintf (stdout, "%.*s ",  clen, underline);
		    break;
	       case GDI_LONG:
	       case GDI_SHORT:
	       case GDI_INTEGER:
		    clen = MAX (LONG_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    fprintf (stdout, "%.*s ", clen, underline);
		    break;
	       default:
		    clen = strlen (GDI_OBJ_COL_NAME(gdi_obj, j));
		    fprintf (stdout, "%.*s ", clen, underline);
	       }
	  }
	       fprintf (stdout, "\n");
     }
     else /* delimited.  comma between fields, strings in "" */
     {
	  underline_len = 0;
	  /*
	   * print column headings separated by commas.
	   */
	  for (j = 0; j < ncols; j++)
	  {
	       if (j > 0 ) 
	       {
		    fprintf (stdout, ", ");
		    underline_len += 2;
	       }
	       
	       fprintf (stdout, "%s", GDI_OBJ_COL_NAME (gdi_obj, j));
	       underline_len += strlen (GDI_OBJ_COL_NAME (gdi_obj, j));
	  }
	  fprintf (stdout, "\n");
	  fprintf (stdout, "%.*s\n", underline_len, underline);
     }	
     
     return (GDI_SUCCESS);
}

/*
 * gdi_print_tuples - the actual data!
 *
 * This calls the print function in the tuple constructor
 *
 */

int
gdi_print_tuples (gdi_obj, format, header)
dbObj    *gdi_obj;      	/* dbobj to print out	                        */
dbFormat format;		/* GDI_FIXED_SPACE or GDI_DELIMITED	        */
int       header;		/* TRUE to print headings.  FALSE for data only */
{
     int   ncols;
     int   ntuples;
     void  *tuple;
     void  *data;
     int   is_null;
     int   str_length;
     int   arr_length;
     int   clen;
     int   i;
     int   j;
     
     if (gdi_obj == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA, 
			 "gdi_print_tuples: NULL dbObj");
	  return (NULL);
     }
     
     if (GDI_OBJ_CONSTRUCTOR (gdi_obj) -> print_data != NULL)
	  return (GDI_OBJ_CONSTRUCTOR (gdi_obj) -> 
		  print_data (gdi_obj, format, header) );

     ntuples = GDI_OBJ_NUM_TUPLES (gdi_obj);
     ncols = gdi_obj_num_columns (gdi_obj);
     
     if ( header == TRUE )
	  gdi_print_tuple_header (gdi_obj, format);

     for (i = 0; i < ntuples; i++)     
     {
	  if ((tuple = gdi_obj_tuple_retrieve (gdi_obj, i)) == NULL)
	       return (GDI_FAILURE);

	  for (j = 0; j < ncols; j++)
	  {
	       if ((data = gdi_obj_get_data (gdi_obj, j, tuple, &is_null,
					     &str_length, &arr_length))
		   == NULL)
		    return (GDI_FAILURE);
	       
	       if (j > 0)
	       {
		    if (format == GDI_DELIMITED)
			 fprintf (stdout, ",");
		    fprintf (stdout, " ");
	       }
	       
	       switch (GDI_OBJ_COL_CTYPE (gdi_obj, j))
	       {
	       case GDI_STRING:
		    if (format == GDI_DELIMITED)
			 fprintf (stdout, "\"%s\"", (char *) data);
		    else
		    {
			 clen = MAX (GDI_OBJ_COL_MAX_STRLEN (gdi_obj, j), 
				     strlen (GDI_OBJ_COL_NAME (gdi_obj, j)));
			 fprintf (stdout, "%-*.*s", clen, clen, (char *) data);
		    }
		    break;
	       case GDI_CHAR:
		    clen = strlen (GDI_OBJ_COL_NAME(gdi_obj, j)) - 1;
		    fprintf (stdout, "%c%*.*s", * (char *) data, clen, clen, " ");
		    break;
	       case GDI_DOUBLE:
	       case GDI_FLOAT:
		    clen = MAX (DBL_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    if (format == GDI_DELIMITED)
			 fprintf (stdout, "%f", * (double *) data);
		    else
			 fprintf (stdout, "%*.*f", clen, DBL_PREC, 
				  * (double *) data);
		    break;
	       case GDI_LONG:
		    clen = MAX (LONG_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    if (format == GDI_DELIMITED)
			 fprintf (stdout, "%d", * (long *) data);
		    else
			 fprintf (stdout, "%*d", clen, * (long *) data);
		    break;
	       case GDI_INTEGER:
		    clen = MAX (LONG_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    if (format == GDI_DELIMITED)
			 fprintf (stdout, "%d", * (int *) data);
		    else
			 fprintf (stdout, "% *d", clen, * (int *) data);
		    break;
	       case GDI_SHORT:
		    clen = MAX (LONG_WIDTH, strlen (GDI_OBJ_COL_NAME (gdi_obj,j)));
		    if (format == GDI_DELIMITED)
			 fprintf (stdout, "%d", * (short *) data);
		    else
			 fprintf (stdout, "% *d", clen, * (short *) data);
		    break;
	       case GDI_UNDEFINED:
	       case GDI_USER_DEFINED:
	       default:
		    clen = strlen (GDI_OBJ_COL_NAME (gdi_obj, j));
		    fprintf (stdout, "%-*.*s", clen, clen, " ");
		    gdi_warning_app (NULL, GDI_BADDATA, 
				     "gdi_print_tuples: Don't know how to print data type");
		    break;
	       }
	  }
	  fprintf (stdout, "\n");
     }

     return (GDI_SUCCESS);
}


