/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_sdi.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *	#include "gdi_sdi.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for creating SDI data for the Generic 
 *      Database Interface.  To understand how the sdi constructor functions 
 *      work, you must first understand how and when the functions are called 
 *      by the other gdi functions such as gdi_submit() and gdi_insert().  See
 *      the GDI Design Manual for these explanations.
 *
 *      The structure of the data container is
 *     
 *      typedef	struct {
 *           int	n_rows;
 *           int	n_cols;
 *           dbColDef	**col_defs;
 *           char	**col_names;
 *           void	**data;
 *      } sdiArray;
 *
 *      col_defs and col_names are null terminated arrays.  col_defs is a copy
 *      of the col_defs field in the dbObj.  col_names contains the names of
 *      the database columns in lower case.  The data in sdiArray is arranged
 *      by column rather than by row.  **data is an array of malloc'd columns.
 *
 *      Since the final data is not arranged in tuples (re rows) the sdi 
 *      constructor functions create a temporary tuple that is needed and
 *      manipulated by the gdi library through the sdi constructor functions.  
 *      When the tuple is added to the container, the data is copied into the data 
 *      arrays.  The tuple is no longer needed after the data has been copied into
 *      the data array.  Only one temporary tuple is needed by the library so the
 *      tuple is malloc'd once by the gdi_sdi_container_create() and placed in the 
 *      args which is a pointer to the dbConstr args.  the dbConstr args are passed
 *      to each gdi_sdi constructor function, so the tuple is always available.
 *      The temporary tuple is destroyed by gdi_sdi_container_close().
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_sdi.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *	gdi_sdi.h
 *
 * AUTHOR
 */


#ifndef	lint
static char SccsId[] = "@(#)gdi_sdi.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

#include <stddef.h>
#include <stdlib.h>
#include <ctype.h>
#include "utilities.h"
#include "libgendb.h"
#include "gdi_sdi.h"

#define SDI_ARRAY_INCR	10


typedef union {
     char	*string_val;
     double	double_val;
} sdiValue;


typedef struct {
     dbColDef	**col_defs;
     sdiValue	data [SDI_MAX_COLUMNS];
} sdiTuple;


/*
 * prototypes for SDI Tuple Constructor
 */

Proto (static sdiArray *, gdi_sdi_array_create,
       (
	sdiTuple   **args               /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_sdi_array_destroy,
       (
	sdiArray   *sArray,		/* (i) sdi data container */
	sdiTuple   **args               /* (i) pointer to constructor arguments */
	));

Proto (static sdiTuple *, gdi_sdi_tuple_create, 
       (
	dbColDef   **col_def, 		/* (i) column defintion */
	sdiTuple   **args               /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_sdi_tuple_destroy, 
       (
	sdiTuple   *tuple, 		/* (i) sdi tuple       */
	sdiTuple   **args               /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_sdi_fill_data, 
       (
	int        col_num, 		/* (i) column number     */
	dbColDef   *col_def, 		/* (i) column definition */
	sdiTuple   *tuple, 		/* (i) sdi tuple         */
	void       *dbdata, 		/* (i) database data     */
	int	   is_null, 		/* (i) "is null" flag    */
	int        str_length,		/* (i) string length     */
	int        array_length,	/* (i) array length      */
	sdiTuple   **args               /* (i) pointer to constructor arguments */
	));

Proto (static void *, gdi_sdi_get_data, 
       (
	int        col_num, 		/* (i) column number     */
	dbColDef   *col_def, 		/* (i) column definition */
	sdiTuple   *tuple, 		/* (i) sdi tuple         */
	int        *is_null, 		/* (o) "is null" flag    */
	int        *str_length,		/* (o) string length       */
	int        *array_length,	/* (o) array length      */
	sdiTuple   **args               /* (o) pointer to constructor arguments */
	));

Proto (static int, gdi_sdi_tuple_add, 
       (
	sdiArray   **sArray,		/* (i) sdi data container */
	sdiTuple   *tuple, 		/* (i) sdi tuple          */
	sdiTuple   **args               /* (i) pointer to constructor arguments */
	));

Proto (static sdiTuple *, gdi_sdi_tuple_retrieve, 
       (
	sdiArray   *sArray, 		/* (i) data container  */
	int        tuple_num,		/* (i) tuple number    */
	sdiTuple   **args               /* (i) pointer to constructor arguments */
	));


#ifdef __STDC__
dbConstr	GDI_SDI_CONSTR = 
{
     (int    (*) (void *, void **))		gdi_sdi_array_destroy,
     (void * (*) (void **))			gdi_sdi_array_create,
     (int    (*) (void **, void *, void **))	gdi_sdi_tuple_add,
     (void * (*) (void *, int, void **))	gdi_sdi_tuple_retrieve,
     (int    (*) (void *, void **))		gdi_sdi_tuple_destroy,
     (void * (*) (dbColDef **, void **))        gdi_sdi_tuple_create,
     (int    (*) (int, dbColDef *, void *, void *, int, int, int, void **))
	                                        gdi_sdi_fill_data,
     (void * (*) (int, dbColDef *, void *, int *, int *, int *, void **)) 
	                                        gdi_sdi_get_data,
     (int    (*) ())	                        NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DBL_STR_COERCE,
};
#else
dbConstr	GDI_SDI_CONSTR = 
{
     (int    (*) ())	gdi_sdi_array_destroy,
     (void * (*) ())	gdi_sdi_array_create,
     (int    (*) ())	gdi_sdi_tuple_add,
     (void * (*) ())	gdi_sdi_tuple_retrieve,
     (int    (*) ())	gdi_sdi_tuple_destroy,
     (void * (*) ())	gdi_sdi_tuple_create,
     (int    (*) ())	gdi_sdi_fill_data,
     (void * (*) ()) 	gdi_sdi_get_data,
     (int    (*) ())	NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DBL_STR_COERCE,
};
#endif



/*
 * gdi_sdi_array_create ()
 *
 * Create a new SDI data container.  Create a single sdiTuple for the dbConstr args.
 *
 * Private
 */
static sdiArray *
gdi_sdi_array_create (args)
sdiTuple   **args;
{
     sdiArray	*sArray;
     char	*routine = "gdi_sdi_array_create: ";
     
     if ((sArray = UCALLOC (sdiArray, 1)) == NULL)
	  gdi_error_unix (NULL, routine);
     
     if (((*args) = UCALLOC (sdiTuple, 1)) == NULL)
     {
	  UFREE (sArray);
	  gdi_error_unix (NULL, routine);
     }

     return (sArray);
}



/*
 * gdi_sdi_array_destroy ()
 *
 * Destroy an SDI data container.  Free all data memory.  Destroy the dbConstr args.
 *
 * Private
 */
static int
gdi_sdi_array_destroy (sArray, args)
sdiArray	*sArray;
sdiTuple        **args;
{
     int	i;
     
     if (sArray == NULL)
	  return (gdi_warning_app (NULL, GDI_BADDATA,
				   "gdi_sdi_array_destroy: NULL sArray argument"));

     /* free data arrays */
     for (i = 0; i < sArray->n_cols; i++)
	  UFREE (sArray->data[i]);
     UFREE (sArray->data);

     /* free column names */
     for (i = 0; i < sArray->n_cols; i++)
	  UFREE (sArray->col_names[i]);
     UFREE (sArray->col_names);
     
     UFREE (*args);
     free (sArray);
     
     return (GDI_SUCCESS);
}



/*
 * gdi_sdi_tuple_create ()
 *
 * Create a new SDI Tuple.  New memory is not allocated since the data will
 * have to be copied into an array in the next step anyway.  The dbConstr
 * args will be used as the temporary sdiTuple.  The args were allocated when
 * the container was created in gdi_sdi_container_create().
 *
 * Private
 */
static sdiTuple *
gdi_sdi_tuple_create (coldef, args)
dbColDef   **coldef;
sdiTuple   **args;
{
     sdiTuple	*tuple = *args;
     
     if (coldef == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			"gdi_sdi_tuple_create: NULL coldef argument");
	  return (NULL);
     }
     
     tuple->col_defs = coldef;

     return (tuple);
}


/*
 * gdi_sdi_tuple_destroy ()
 *
 * Destroy an SDI tuple.  Only strings values are allocated dynamically.
 *
 * Private
 */
static int
gdi_sdi_tuple_destroy (tuple, args)
sdiTuple	*tuple;
sdiTuple        **args;
{
     int	i;
     
     if (tuple == NULL)
	  return (gdi_warning_app (NULL, GDI_BADDATA,
				   "gdi_sdi_tuple_destroy: NULL tuple argument"));
     
     for (i = 0; (tuple->col_defs[i] != NULL) && (i < SDI_MAX_COLUMNS); i++)
	  if (tuple->col_defs[i]->ctype == GDI_STRING)
	       UFREE (tuple->data[i].string_val);
     
     return (GDI_SUCCESS);
}     
	  
     
/*
 * gdi_sdi_fill_data ()
 *
 * Fill an SDI tuple column value with database data.  Length for a string
 * may include a NULL terminator.
 *
 * Private
 */
static int
gdi_sdi_fill_data (col_num, col_def, tuple, dbdata, is_null, 
		   str_length, arr_length, args)
int		col_num;
dbColDef	*col_def;
sdiTuple	*tuple;
void		*dbdata;
int		is_null;
int		str_length;
int		arr_length;
sdiTuple        **args;
{
     char	*routine = "gdi_sdi_fill_data: ";

     if ((col_num < 0) || (col_num > (SDI_MAX_COLUMNS - 1)))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_fill_data: invalid column number");
	  return (GDI_FAILURE);
     }
     
     if ((col_def == NULL) || (tuple == NULL) || (dbdata == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_fill_data: NULL pointer argument");
	  return (GDI_FAILURE);
     }
     
     if (arr_length > 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_fill_data: Can't handle array column data");
	  return (GDI_FAILURE);
     }
     
     switch (col_def->ctype)
     {
     case GDI_STRING:
	  if ((tuple->data[col_num].string_val = UCALLOC (char, str_length + 1)) == NULL)
	  {
	       gdi_error_unix (NULL, routine);
	       return (GDI_FAILURE);
	  }
	  bcopy ((char *) dbdata, tuple->data[col_num].string_val, 
		str_length * sizeof (char));
	  tuple->data[col_num].string_val [str_length] = '\0';
	  break;
     case GDI_DOUBLE:
	  tuple->data[col_num].double_val = * ((double *) dbdata);
	  break;
     default:
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_fill_data: ctype not a string or a double");
	  return (GDI_FAILURE);
	  break;
     }
     
     return (GDI_SUCCESS);
}


/*
 * gdi_sdi_get_data ()
 *
 * Return an SDI tuple column value.  Also fill in is_null and length.
 *
 * Private
 */
static void *
gdi_sdi_get_data (col_num, col_def, tuple, is_null, str_length, arr_length, args)
int		col_num;
dbColDef	*col_def;
sdiTuple	*tuple;
int		*is_null;
int		*str_length;
int		*arr_length;
sdiTuple        **args;
{
     void	*dbdata;
     
     if ((col_num < 0) || (col_num > (SDI_MAX_COLUMNS - 1)))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_get_data: invalid column number");
	  return (NULL);
     }
     
     if ((col_def == NULL) || (tuple == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_get_data: NULL pointer argument");
	  return (NULL);
     }

     if (is_null != NULL)
	  *is_null = 0;
     
     if (arr_length != NULL)
	  *arr_length = GDI_SCALAR;
     
     if (str_length != NULL)
     {
	  if (col_def->ctype == GDI_STRING)
	       *str_length = strlen (tuple->data[col_num].string_val);
	  else
	       *str_length = 0;
     }
     
     if (col_def->ctype == GDI_STRING)
	  dbdata = (void *) tuple->data[col_num].string_val;
     else
	  dbdata = (void *) &(tuple->data[col_num]);
     
     return (dbdata);
}



/*
 * gdi_sdi_tuple_add ()
 *
 * Add an SDI tuple to the SDI tuple container
 *
 * Private
 */
static int
gdi_sdi_tuple_add (sArray, tuple, args)
sdiArray	**sArray;
sdiTuple	*tuple;
sdiTuple        **args;
{
     int	ncols;
     int	nrows;
     int	i, j;
     void	*p;
     char	*routine = "gdi_sdi_tuple_add: ";
     
     if ((tuple == NULL) || (sArray == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_tuple_add: NULL pointer argument");
	  return (GDI_FAILURE);
     }

     if (*sArray == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_tuple_add: SDI array not allocated");
	  return (GDI_FAILURE);
     }

     /*
      * if this is the first time the routine is called, data arrays
      * will not have been allocated yet
      */
     if ((*sArray)->data == NULL)
     {
	  if (tuple->col_defs == NULL)
	  {
	       gdi_error_app (NULL, GDI_BADDATA,
			      "gdi_sdi_tuple_add: NULL col defs");
	       return (GDI_FAILURE);
	  }

	  /* calculate the number of columns */
	  for (ncols = 0; tuple->col_defs[ncols] != NULL; ncols++);

	  /* allocate the NULL terminated array of data columns */
	  if (((*sArray)->data = UCALLOC (void *, ncols + 1)) == NULL)
	  {
	       gdi_error_unix (NULL, routine);
	       return (GDI_FAILURE);
	  }

	  /* allocate the NULL terminated array of column name strings */
	  if (((*sArray)->col_names = UCALLOC (char *, ncols + 1)) == NULL)	  
	  {
	       gdi_error_unix (NULL, routine);
	       UFREE ((*sArray)->data);
	       return (GDI_FAILURE);
	  }
	  for (i = 0; i < ncols; i++)
	  {
	       if (((*sArray)->col_names[i] = UCALLOC (char,
						       GDI_MAX_NAME + 1)) == NULL)
	       {
		    gdi_error_unix (NULL, routine);
		    UFREE ((*sArray)->data);
		    for (i--; i >= 0; i--)
			 UFREE ((*sArray)->col_names[i]);
		    UFREE ((*sArray)->col_names);
		    return (GDI_FAILURE);
	       }
	       for (j = 0; j < (GDI_MAX_NAME + 1); j++)
		    (*sArray)->col_names[i][j] = tolower (tuple->col_defs[i]->name[j]);
	  }

	  (*sArray)->n_cols = ncols;
	  (*sArray)->col_defs = tuple->col_defs;

     }
     
     /*
      * if number of rows is a multiple of SDI_ARRAY_INCR, then
      * reallocate data arrays (increase by SDI_ARRAY_INCR)
      */
     nrows = (*sArray)->n_rows;
     if ((nrows % SDI_ARRAY_INCR) == 0)
     {
	  for (i = 0; i < (*sArray)->n_cols; i++)
	  {
	       switch (tuple->col_defs[i]->ctype)
	       {
	       case GDI_STRING:
		    if ((p = (void *) UREALLOC ((*sArray)->data[i], char *, 
						nrows + SDI_ARRAY_INCR)) 
			== NULL)
		    {
			 gdi_error_unix (NULL, routine);
			 return (GDI_FAILURE);
		    }	
		    break;
	       default:
		    if ((p = (void *) UREALLOC ((*sArray)->data[i], double, 
				       nrows + SDI_ARRAY_INCR)) 
			== NULL)
		    {
			 gdi_error_unix (NULL, routine);
			 return (GDI_FAILURE);
		    }
		    break;
	       }
	       (*sArray)->data[i] = p;
	  }
	  
     }

     /*
      * copy data from tuple into data arrays
      */
     for (i = 0; i < (*sArray)->n_cols; i++)
     {
	  switch (tuple->col_defs[i]->ctype)
	  {
	  case GDI_STRING:
	       ((char **) (*sArray)->data[i])[nrows] = tuple->data[i].string_val;
	       break;
	  case GDI_DOUBLE:
	       ((double *) (*sArray)->data[i])[nrows] = tuple->data[i].double_val;
	       break;
	  default:
	       break;
	  }
     }
     
     (*sArray)->n_rows++;
     return (GDI_SUCCESS);
}


/*
 * gdi_sdi_tuple_retrieve ()
 *
 * Retrieve an SDI tuple from the SDI tuple container
 *
 * Private
 */
static sdiTuple *
gdi_sdi_tuple_retrieve (sArray, tuple_num, args)
sdiArray	*sArray;
int		tuple_num;
sdiTuple        **args;
{
     sdiTuple	*tuple = *args;
     int	i;
     
     if (sArray == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_tuple_retrieve: NULL sArray argument");
	  return (NULL);
     }

     if ((tuple_num < 0) || (tuple_num >= sArray->n_rows))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_tuple_retrieve: tuple_num out of range");
	  return (NULL);
     }

     if ((tuple->col_defs = sArray->col_defs) == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_sdi_tuple_retrieve: sArray->col_defs not defined");
	  return (NULL);
     }

     /*
      * copy data from data arrays into tuple
      */
     for (i = 0; i < sArray->n_cols; i++)
     {
	  switch (tuple->col_defs[i]->ctype)
	  {
	  case GDI_STRING:
	       tuple->data[i].string_val = ((char **) sArray->data[i])[tuple_num];
	       break;
	  case GDI_DOUBLE:
	       tuple->data[i].double_val = ((double *) sArray->data[i])[tuple_num];
	       break;
	  default:
	       break;
	  }
     }
     
     /*
      * return pointer to tuple
      */
     return (tuple);
}

