/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_begin_tran()	begin a transaction (SYBASE & POSTGRES)
 *	gdi_auto_commit()	toggle auto commit and ansi tm (ORACLE only)
 *	gdi_commit()		end a transaction, making changes permanent
 *	gdi_rollback()		end a transaction, discarding changes
 *	gdi_savepoint()		set a savepoint (SYBASE & ORACLE)
 *
 * DESCRIPTION
 *	This file contains functions for the database transaction management.
 *
 * DIAGNOSTICS
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	gdi_tran.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_tran.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_commit ()
 *
 * Commit a database transaction.
 *
 * Public
 */

int
gdi_commit (conn, channo)
dbConn	*conn;
int	channo;
{
     int    ret;
     char   *fname = "gdi_commit: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_commit: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->commit) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->commit (conn, channo)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_commit: Vendor function unavailable");

     return (GDI_FAILURE);
}



/*
 * gdi_begin_tran ()
 *
 * Start a transaction. Relevant only to SYBASE and POSTGRES.
 *
 * Public
 */

int
gdi_begin_tran (conn, channo, tran_name)
dbConn	*conn;
int	channo;
char	*tran_name;
{
     int    ret;
     char   *fname = "gdi_begin_tran: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_begin_tran: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->begin_tran) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->begin_tran (conn, channo, tran_name)) 
	      != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_warning_app (conn, GDI_BADVENDOR, 
		    "gdi_begin_tran: Vendor function unavailable");

     return (GDI_FAILURE);
}

/*
 * gdi_rollback ()
 *
 * Rollback a database transaction.
 *
 * Public
 */

int
gdi_rollback (conn, channo)
dbConn	*conn;
int	channo;
{
     int    ret;
     char   *fname = "gdi_rollback: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_rollback: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->rollback) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->rollback (conn, channo)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_rollback: Vendor function unavailable");

     return (GDI_FAILURE);
}



/*
 * gdi_savepoint ()
 *
 * Set a "save" point for a database transaction.
 *
 * Public
 */

int
gdi_savepoint (conn, channo, sname)
dbConn	*conn;
int	channo;
char	*sname;
{
     int    ret;
     char   *fname = "gdi_save_point: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_savepoint: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->savepoint) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->savepoint (conn, channo, sname)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_savepoint: Vendor function unavailable");

     return (GDI_FAILURE);
}
